/* $Log: ftest.c,v $
 * Revision 1.10  1993/03/10  06:26:49  marcusm
 * Added exit(0) at end.  Was returning a non-zero status for some reason.
 *
 * Revision 1.9  1993/01/28  20:52:04  marcusm
 * Commented out 'tan' test that was always getting a loss of precision
 *
 */
#ifndef _NAM
#   define _wait;
#   define pipepipe
#else
#   include "rts-nam.h"
#endif

#if 0
***************************************************************************
*  Copyright 1985 MetaWare Incorporated.  All rights reserved.		      *
*  This is an unpublished work, Copyright 1985, by MetaWare Incorporated. *
*  Use, disclosure, or transfer without the express written permission	  *
*  of MetaWare Incorporated is strictly prohibited.			              *
*  If any or part of this source code is incorporated into other source   *
*  code not authored by MetaWare, this copyright notice must          	  *
*  accompany it.							                              *
***************************************************************************
*  This part of the test suite is a separate program that does a spot	  *
*  check on the floating-point math library functions.			          *
*  It produces a number of lines reading "Testing fff" where fff is a     *
*  library function.  Only if the library function is not accurate	      *
*  enough does it print out further information.  For example, here is	  *
*  a sample run with some inaccuracies: 				                  *
Testing acos
Testing asin
Testing atan
Testing tan
Diff = 7.582912076031789e-11 Eps = 1.0e-12
tan(  -1.568296332003210e+00)=-3.999999999999242e+02;
	   correct answer is -4.000000000000000e+02; oops!.
Diff = 3.979039320256561e-13 Eps = 1.0e-13
tan(   1.554131203080956e+00)=6.000000000000040e+01;
	   correct answer is 6.000000000000000e+01; oops!.
Diff = 8.742517820792273e-10 Eps = 1.0e-11
tan(   1.570296326836563e+00)=1.999999999999126e+03;
	   correct answer is 2.000000000000000e+03; oops!.
Testing sin
Testing cos
Testing cosh
Testing sinh
Diff = 4.108109794183357e-20 Eps = 1.0e-20
sinh(  -1.231887900000000e-06)=-1.231887900000352e-06;
	   correct answer is -1.231887900000311e-06; oops!.
Diff = 4.108109794183357e-20 Eps = 1.0e-20
sinh(	1.231887900000000e-06)=1.231887900000352e-06;
	   correct answer is 1.231887900000311e-06; oops!.
Testing tanh
Diff = 4.044582323139284e-20 Eps = 1.0e-20
tanh(  -1.231887900000000e-06)=-1.231887899999417e-06;
	   correct answer is -1.231887899999377e-06; oops!.
Diff = 4.044582323139284e-20 Eps = 1.0e-20
tanh(	1.231887900000000e-06)=1.231887899999417e-06;
	   correct answer is 1.231887899999377e-06; oops!.
Diff = 2.710505431213761e-20 Eps = 1.0e-20
tanh(  -1.000000000000000e-05)=-9.999999999666695e-06;
	   correct answer is -9.999999999666668e-06; oops!.
Testing sqrt
Testing floor
Testing fabs
Testing ceil
Testing log10
Testing log
Testing exp
Testing atan2
Testing pow
Testing fmod
Testing ldexp
Testing frexp
Testing modf

*  In each error case, Eps is the desired difference between the supplied *
*  answer and the obtained answer, and Diff is the actual result.	  *
*  The obtained and supplied answer are both listed.			  *
***************************************************************************
#endif

#include <stdio.h>
#include <errno.h>
#include "which.h"

#ifdef HIGHC
// pragma off(floating_point); /* was getting warning. WHat is this? DJW */
#pragma on(emit_line_records);
#endif

/* Spot check of the math functions. */

#ifdef PROTOTYPE
typedef double Double_to_double(double);
typedef double Double2_to_double(double,double);
typedef double type_of_ldexp(double,int);
typedef double type_of_frexp(double,int*);
typedef double type_of_modf(double,double*);
#else
typedef double Double_to_double();
typedef double Double2_to_double();
typedef double type_of_ldexp();
typedef double type_of_frexp();
typedef double type_of_modf();
#endif

extern Double_to_double acos, asin, atan, cos, sin, tan, cosh, sinh, tanh;
extern Double_to_double log10, fabs, floor, ceil, sqrt, log, exp;
extern Double2_to_double atan2, pow, fmod;
extern type_of_ldexp ldexp;
extern type_of_frexp frexp;
extern type_of_modf modf;

/* double -> double. */
typedef struct test {double Arg, Answer; int Agree_after_dot;} tv[];
/* double, double -> double. */
typedef struct test2 {double Arg1, Arg2, Answer; int Agree_after_dot;} tv2[];
/* double, int -> double. */
typedef struct test3 {double Arg1; int Arg2; double Answer; int Agree_after_dot;} tv3[];
/* double, *int -> double. */
typedef struct test4 {double Arg1; double Answer1; int Answer2, Agree_after_dot;} tv4[];
/* double, *double -> double. */
typedef struct test5 {double Arg1; double Answer1, Answer2; int Agree_after_dot;} tv5[];

static Boolean debug,pv;


double NP10[40]; /* NP10[i] = 10**(-i). */

void test(fname, fn, V, testcount, Inverse)
   char *fname; Double_to_double *fn;
   tv V; int testcount; Boolean Inverse; {
   int i;
   if (pv) printf("static tv %s_vector = {\n",fname);
   else printf("Testing %s\n",fname);
   for (i = 0; i < testcount; i++) {
      double Arg = V[i].Arg, Answer = V[i].Answer;
      double Fcn_result, Temp, Diff, Eps;
      Boolean oops;
      int Digits = V[i].Agree_after_dot - 1;
      if (Inverse) { Temp = Arg; Arg = Answer; Answer = Temp; }
      Fcn_result = (*fn)(Arg);
      Diff = Fcn_result-Answer;
      if (Diff < 0) Diff = -Diff;
      /* Scale the difference to reflect the size of the result. */
      /* Ensure accuracy to Digits digits. */
      /* Don't waste time doing pow, log, etc. if Diff = 0. */
      oops = (Eps = 0, Diff != 0) && Diff >
	 (Eps = NP10[Digits-1]*
	 (Answer == 0? 1 : pow(10.0,floor(log10(fabs(Answer))))));
      if ((debug || oops) && !pv) {
	 printf("Diff = %.15e Eps = %.1e\n",Diff,Eps);
	 printf("%s(%24.15e)=%.15e; \n           correct answer is %.15e; %s.\n",
		fname,Arg,Fcn_result,Answer,
		oops?"oops!":"OK");
	 }
      if (pv)
	 printf("   %24.15e,  %24.15e, %2d,\n",Arg, Fcn_result, Digits);
      }
   if (pv) printf("};\n");
   }


void test2(fname, fn, V, testcount)
   char *fname; Double2_to_double *fn; tv2 V; int testcount; {
   int i;
   if (pv) printf("static tv2 %s_vector = {\n",fname);
   else printf("Testing %s\n",fname);
   for (i = 0; i < testcount; i++) {
      double Arg1 = V[i].Arg1, Arg2 = V[i].Arg2, Answer = V[i].Answer;
      double Fcn_result, Diff, Eps;
      Boolean oops;
      int Digits = V[i].Agree_after_dot - 1;
      errno = 0;
      Fcn_result = (*fn)(Arg1,Arg2);
    /* Microsoft C doesn't do pow correctly:  -x**-n never works, even
       when n is an integer.  Correct for that here: */

      if (fn == pow && Fcn_result == 0 && Arg1 < 0 && errno != 0) {
         Fcn_result = (*fn)(fabs(Arg1),Arg2);
         if ((int)Arg2 & 1) Fcn_result = -Fcn_result;
         }
      Diff = Fcn_result-Answer;
      if (Diff < 0) Diff = -Diff;
      /* Scale the difference to reflect the size of the result. */
      /* Ensure accuracy to Digits digits. */
      /* Don't waste time doing pow, log, etc. if Diff = 0. */
      Eps = NP10[Digits-1];
      oops = (!pv) && (Diff > Eps);
      if ((debug || oops) && !pv) {
         printf("Diff = %.15e Eps = %.1e\n",Diff,Eps);
         printf("%s(%24.15e,%24.15e)=%.15e; \n           correct answer is %.15e; %s.\n",
                fname,Arg1,Arg2,Fcn_result,Answer,
                oops?"oops!":"OK");
         }
      if (pv)
         printf("   %24.15e,  %24.15e, %24.15e, %2d,\n",Arg1, Arg2, Fcn_result, Digits);
      }
   if (pv) printf("};\n");
   }

void test3(fname, fn, V, testcount)
   char *fname; type_of_ldexp *fn; tv3 V; int testcount; {
   int i;
   if (pv) printf("static tv3 %s_vector = {\n",fname);
   else printf("Testing %s\n",fname);
   for (i = 0; i < testcount; i++) {
      double Arg1 = V[i].Arg1; int Arg2 = V[i].Arg2; double Answer = V[i].Answer;
      double Fcn_result, Diff, Eps;
      Boolean oops;
      int Digits = V[i].Agree_after_dot - 1;
      errno = 0;
      Fcn_result = (*fn)(Arg1,Arg2);
      Diff = Fcn_result-Answer;
      if (Diff < 0) Diff = -Diff;
      /* Scale the difference to reflect the size of the result. */
      /* Ensure accuracy to Digits digits. */
      /* Don't waste time doing pow, log, etc. if Diff = 0. */
      oops = (!pv) && (Eps = 0, Diff != 0) && Diff >
         (Eps = NP10[Digits-1]*
         (Answer == 0? 1 : pow(10.0,floor(log10(fabs(Answer))))));
      if ((debug || oops) && !pv) {
         printf("Diff = %.15e Eps = %.1e\n",Diff,Eps);
         printf("%s(%24.15e,%d)=%.15e; \n           correct answer is %.15e; %s.\n",
                fname,Arg1,Arg2,Fcn_result,Answer,
                oops?"oops!":"OK");
         }
      if (pv)
         printf("   %24.15e,  %5d, %24.15e, %2d,\n",Arg1, Arg2, Fcn_result, Digits);
      }
   if (pv) printf("};\n");
   }

void test4(fname, fn, V, testcount)
   char *fname; type_of_frexp *fn; tv4 V; int testcount; {
   int i;
   if (pv) printf("static tv4 %s_vector = {\n",fname);
   else printf("Testing %s\n",fname);
   for (i = 0; i < testcount; i++) {
      double Arg1 = V[i].Arg1; double Answer1 = V[i].Answer1;
      int Answer2 = V[i].Answer2;
      double Fcn_result, Diff, Eps;
      Boolean oops;
      int Digits = V[i].Agree_after_dot - 1, Result2;
      errno = 0;
      Fcn_result = (*fn)(Arg1,&Result2);
      Diff = Fcn_result-Answer1;
      if (Diff < 0) Diff = -Diff;
      /* Scale the difference to reflect the size of the result. */
      /* Ensure accuracy to Digits digits. */
      /* Don't waste time doing pow, log, etc. if Diff = 0. */
      oops = (!pv) && ((Answer2 != Result2) || (Eps = 0, Diff != 0) && Diff >
         (Eps = NP10[Digits-1]*
         (Answer1 == 0? 1 : pow(10.0,floor(log10(fabs(Answer1)))))));
      if ((debug || oops) && !pv) {
         printf("Diff = %.15e Eps = %.1e\n",Diff,Eps);
         printf("%s(%24.15e)=%.15e, %d; \n           correct answer is %.15e,%d; %s.\n",
                fname,Arg1,Fcn_result,Result2,Answer1,Answer2,
                oops?"oops!":"OK");
         }
      if (pv)
         printf("   %24.15e,  %24.15e, %5d, %2d,\n",Arg1, Fcn_result, Result2, Digits);
      }
   if (pv) printf("};\n");
   }

void test5(fname, fn, V, testcount)
   char *fname; type_of_modf *fn; tv5 V; int testcount; {
   int i;
   if (pv) printf("static tv5 %s_vector = {\n",fname);
   else printf("Testing %s\n",fname);
   for (i = 0; i < testcount; i++) {
      double Arg1 = V[i].Arg1; double Answer1 = V[i].Answer1;
      double Answer2 = V[i].Answer2;
      double Fcn_result, Diff, Eps;
      Boolean oops;
      int Digits = V[i].Agree_after_dot - 1; double Result2;
      errno = 0;
      Fcn_result = (*fn)(Arg1,&Result2);
      Diff = Fcn_result-Answer1;
      if (Diff < 0) Diff = -Diff;
      /* Scale the difference to reflect the size of the result. */
      /* Ensure accuracy to Digits digits. */
      /* Don't waste time doing pow, log, etc. if Diff = 0. */
      oops = (!pv) && ((Answer2 != Result2) || (Eps = 0, Diff != 0) && Diff >
	 (Eps = NP10[Digits-1]*
	 (Answer1 == 0? 1 : pow(10.0,floor(log10(fabs(Answer1)))))));
      if ((debug || oops) && !pv) {
	 printf("Diff = %.15e Eps = %.1e\n",Diff,Eps);
	 printf("%s(%24.15e)=%.15e, %.15e; \n           correct answer is %.15e,%.15e; %s.\n",
		fname,Arg1,Fcn_result,Result2,Answer1,Answer2,
		oops?"oops!":"OK");
	 }
      if (pv)
	 printf("   %24.15e,  %24.15e, %24.15e, %2d,\n",Arg1, Fcn_result, Result2, Digits);
      }
   if (pv) printf("};\n");
   }

/* The 16-digit numbers were derived by using another, working, compiler. */
static tv acos_vector = {
    -1.000000000000000e+000,	3.141592653589793e+000, 15,
    -9.800000000000000e-001,	2.941257811266674e+000, 15,
    -8.863131229100000e-001,	2.660118277774513e+000, 15,
    -8.000000000000000e-001,	2.498091544796509e+000, 15,
    -5.000000000000000e-001,	2.094395102393196e+000, 15,
    -7.732343212000000e-001,	2.454722221467730e+000, 15,
    -2.000000000000000e-001,	1.772154247585227e+000, 15,
     0.000000000000000e+000,	1.570796326794897e+000, 15,
     2.000000000000000e-001,	1.369438406004566e+000, 15,
     5.503000000000000e-001,	9.880728358170411e-001, 15,
     3.388123120000000e-001,	1.225142067693980e+000, 15,
     7.754600000000000e-001,	6.833530138154190e-001, 15,
     9.998000000000000e-001,	2.000033334833312e-002, 15,
     1.000000000000000e+000,	0.000000000000000e+000, 15,
};
static tv asin_vector = {
    -1.000000000000000e+000,   -1.570796326794897e+000, 15,
    -9.800000000000000e-001,   -1.370461484471777e+000, 15,
    -8.000000000000000e-001,   -9.272952180016123e-001, 15,
    -7.732343212000000e-001,   -8.839258946728336e-001, 15,
    -5.000000000000000e-001,   -5.235987755982989e-001, 15,
    -2.000000000000000e-001,   -2.013579207903308e-001, 15,
     0.000000000000000e+000,	0.000000000000000e+000, 15,
     2.000000000000000e-001,	2.013579207903308e-001, 15,
     5.503000000000000e-001,	5.827234909778556e-001, 15,
     7.754600000000000e-001,	8.874433129794775e-001, 15,
     3.388123120000000e-001,	3.456542591009162e-001, 15,
     9.998000000000000e-001,	1.550795993446564e+000, 15,
     1.000000000000000e+000,	1.570796326794897e+000, 15,
};
static tv atan_vector = {
    -1.000000000000000e+000, -7.853981633974483e-001, 15,
    -9.800000000000000e-001, -7.752974968121263e-001, 15,
    -8.000000000000000e-001, -6.747409422235527e-001, 15,
    -8.323231000000000e-001, -6.941417766262270e-001, 15,
    -5.000000000000000e-001, -4.636476090008061e-001, 15,
    -2.000000000000000e-001, -1.973955598498808e-001, 15,
    -1.000000000000000e-005, -9.999999999666668e-006, 15,
     0.000000000000000e+000,  0.000000000000000e+000, 15,
     2.000000000000000e-001,  1.973955598498808e-001, 15,
     5.503000000000000e-001,  5.030735080454593e-001, 15,
     7.754600000000000e-001,  6.595973920697915e-001, 15,
/*  -4.000000000000000e+002, -1.568296332003210e+000, 15,*/
    -3.999999999999330e+002, -1.568296332003210e+000, 15,
     3.771102182312300e-001,  3.606194398808928e-001, 15,
     9.998000000000000e-001,  7.852981533967817e-001, 15,
     1.000000000000000e+000,  7.853981633974483e-001, 15,
    -2.000000000000000e+001, -1.520837931072954e+000, 15,
     6.000000000000000e+001,  1.554131203080956e+000, 15,
/*   2.000000000002880e+003,  1.570296326836564e+000, 15,*/
/*   2.000000000000000e+003,  1.570296326836563e+000, 15,*/
};
static tv cosh_vector = {
    -2.033232320000000e+001,	3.382094781916349e+008, 15,
    -9.123120000000000e+000,	4.582375594292338e+003, 15,
    -3.342112340000000e+000,	1.415708013914054e+001, 15,
    -1.331242300000000e+000,	2.024946200918081e+000, 15,
    -7.783123483200000e-001,	1.318487049586983e+000, 15,
    -2.342340032000000e-003,	1.000002743279667e+000, 15,
    -1.231887900000000e-006,	1.000000000000759e+000, 15,
     0.000000000000000e+000,	1.000000000000000e+000, 15,
     7.712838210000000e-001,	1.312479905746910e+000, 15,
     2.033232320000000e+001,	3.382094781916349e+008, 15,
     9.123120000000000e+000,	4.582375594292338e+003, 15,
     3.342112340000000e+000,	1.415708013914054e+001, 15,
     1.331242300000000e+000,	2.024946200918081e+000, 15,
     7.783123483200000e-001,	1.318487049586983e+000, 15,
     1.772312223120000e+000,	3.027191693093886e+000, 15,
     2.342340032000000e-003,	1.000002743279667e+000, 15,
     1.231887900000000e-006,	1.000000000000759e+000, 15,
    -1.000000000000000e+000,	1.543080634815244e+000, 15,
    -9.800000000000000e-001,	1.519883670390408e+000, 15,
    -8.000000000000000e-001,	1.337434946304845e+000, 15,
    -8.323231000000000e-001,	1.366845009028152e+000, 15,
    -5.000000000000000e-001,	1.127625965206381e+000, 15,
    -2.000000000000000e-001,	1.020066755619076e+000, 15,
    -1.000000000000000e-005,	1.000000000050000e+000, 15,
     0.000000000000000e+000,	1.000000000000000e+000, 15,
     2.000000000000000e-001,	1.020066755619076e+000, 15,
     5.503000000000000e-001,	1.155274911587230e+000, 15,
     7.754600000000000e-001,	1.316041367620252e+000, 15,
     3.771102182312300e-001,	1.071952741733445e+000, 15,
     9.998000000000000e-001,	1.542845625436561e+000, 15,
     1.000000000000000e+000,	1.543080634815244e+000, 15,
    -2.000000000000000e+001,	2.425825977048951e+008, 15,
     6.000000000000000e+001,	5.710036949078421e+025, 15,
};

static tv tanh_vector = {
    -2.033232320000000e+001,   -1.000000000000000e+000, 15,
    -9.123120000000000e+000,   -9.999999761883964e-001, 15,
    -3.342112340000000e+000,   -9.975021557076290e-001, 15,
    -1.331242300000000e+000,   -8.695526305444699e-001, 15,
    -7.783123483200000e-001,   -6.517369705676921e-001, 15,
    -2.342340032000000e-003,   -2.342335748215504e-003, 15,
    -1.231887900000000e-006,   -1.231887899999377e-006, 15,
     0.000000000000000e+000,	0.000000000000000e+000, 15,
     7.712838210000000e-001,	6.476753497016841e-001, 15,
     2.033232320000000e+001,	1.000000000000000e+000, 15,
     9.123120000000000e+000,	9.999999761883964e-001, 15,
     3.342112340000000e+000,	9.975021557076290e-001, 15,
     1.331242300000000e+000,	8.695526305444699e-001, 15,
     7.783123483200000e-001,	6.517369705676921e-001, 15,
     1.772312223120000e+000,	9.438622944428717e-001, 15,
     2.342340032000000e-003,	2.342335748215504e-003, 15,
     1.231887900000000e-006,	1.231887899999377e-006, 15,
    -1.000000000000000e+000,   -7.615941559557649e-001, 15,
    -9.800000000000000e-001,   -7.530659048695520e-001, 15,
    -8.000000000000000e-001,   -6.640367702678490e-001, 15,
    -8.323231000000000e-001,   -6.817214292224805e-001, 15,
    -5.000000000000000e-001,   -4.621171572600097e-001, 15,
    -2.000000000000000e-001,   -1.973753202249040e-001, 15,
    -1.000000000000000e-005,   -9.999999999666668e-006, 15,
     0.000000000000000e+000,	0.000000000000000e+000, 15,
     2.000000000000000e-001,	1.973753202249040e-001, 15,
     5.503000000000000e-001,	5.007450212823495e-001, 15,
     7.754600000000000e-001,	6.500931374008527e-001, 15,
     3.771102182312300e-001,	3.601952297853749e-001, 15,
     9.998000000000000e-001,	7.615101482926131e-001, 15,
     1.000000000000000e+000,	7.615941559557649e-001, 15,
    -2.000000000000000e+001,   -1.000000000000000e+000, 15,
     6.000000000000000e+001,	1.000000000000000e+000, 15,
};

static tv sinh_vector = {
    -2.033232320000000e+001,   -3.382094781916349e+008, 15,
    -9.123120000000000e+000,   -4.582375485178627e+003, 15,
    -3.342112340000000e+000,   -1.412171795731835e+001, 15,
    -1.331242300000000e+000,   -1.760797295719348e+000, 15,
    -7.783123483200000e-001,   -8.593067554305546e-001, 15,
    -2.342340032000000e-003,   -2.342342173897536e-003, 15,
    -1.231887900000000e-006,   -1.231887900000311e-006, 15,
     0.000000000000000e+000,	0.000000000000000e+000, 15,
     7.712838210000000e-001,	8.500608819310631e-001, 15,
     2.033232320000000e+001,	3.382094781916349e+008, 15,
     9.123120000000000e+000,	4.582375485178627e+003, 15,
     3.342112340000000e+000,	1.412171795731835e+001, 15,
     1.331242300000000e+000,	1.760797295719348e+000, 15,
     7.783123483200000e-001,	8.593067554305546e-001, 15,
     1.772312223120000e+000,	2.857252097161997e+000, 15,
     2.342340032000000e-003,	2.342342173897536e-003, 15,
     1.231887900000000e-006,	1.231887900000311e-006, 15,
    -1.000000000000000e+000,   -1.175201193643801e+000, 15,
    -9.800000000000000e-001,   -1.144572571539009e+000, 15,
    -8.000000000000000e-001,   -8.881059821876230e-001, 15,
    -8.323231000000000e-001,   -9.318075330802857e-001, 15,
    -5.000000000000000e-001,   -5.210953054937474e-001, 15,
    -2.000000000000000e-001,   -2.013360025410940e-001, 15,
    -1.000000000000000e-005,   -1.000000000016667e-005, 15,
     0.000000000000000e+000,	0.000000000000000e+000, 15,
     2.000000000000000e-001,	2.013360025410940e-001, 15,
     5.503000000000000e-001,	5.784981601897118e-001, 15,
     7.754600000000000e-001,	8.555494616255587e-001, 15,
     3.771102182312300e-001,	3.861122641277409e-001, 15,
     9.998000000000000e-001,	1.174892601018805e+000, 15,
     1.000000000000000e+000,	1.175201193643801e+000, 15,
    -2.000000000000000e+001,   -2.425825977048951e+008, 15,
     6.000000000000000e+001,	5.710036949078421e+025, 15,
};
static tv sqrt_vector = {
     0.000000000000000e+000,	0.000000000000000e+000, 15,
     7.712838210000000e-001,	8.782276589814284e-001, 15,
     2.033232320000000e+001,	4.509137744624797e+000, 15,
#if 0
     7.000000000000000e+200,	2.645751311064591e+100, 15,
#endif
     9.123120000000000e+000,	3.020450297554985e+000, 15,
     3.342112340000000e+000,	1.828144507417288e+000, 15,
     1.331242300000000e+000,	1.153794739110904e+000, 15,
     1.002121212000000e+004,	1.001060044153197e+002, 15,
     7.783123483200000e-001,	8.822201246401036e-001, 15,
     1.772312223120000e+000,	1.331282172614056e+000, 15,
     2.342340032000000e-003,	4.839772754995837e-002, 15,
     1.231887900000000e-006,	1.109904455347396e-003, 15,
     0.000000000000000e+000,	0.000000000000000e+000, 15,
     2.000000000000000e-001,	4.472135954999579e-001, 15,
     5.503000000000000e-001,	7.418220810949213e-001, 15,
     7.754600000000000e-001,	8.806020667702297e-001, 15,
     3.771102182312300e-001,	6.140930045451014e-001, 15,
     9.998000000000000e-001,	9.998999949995000e-001, 15,
     1.000000000000000e+000,	1.000000000000000e+000, 15,
     7.732232000000000e+006,	2.780689123221076e+003, 15,
     6.000000000000000e+001,	7.745966692414834e+000, 15,
};
static tv floor_vector = {
    -2.033232320000000e+001,   -2.100000000000000e+001, 15,
    -9.123120000000000e+000,   -1.000000000000000e+001, 15,
    -3.342112340000000e+000,   -4.000000000000000e+000, 15,
    -1.331242300000000e+000,   -2.000000000000000e+000, 15,
    -7.783123483200000e-001,   -1.000000000000000e+000, 15,
    -2.342340032000000e-003,   -1.000000000000000e+000, 15,
    -1.231887900000000e-006,   -1.000000000000000e+000, 15,
     0.000000000000000e+000,	0.000000000000000e+000, 15,
     7.712838210000000e-001,	0.000000000000000e+000, 15,
     2.033232320000000e+001,	2.000000000000000e+001, 15,
     9.123120000000000e+000,	9.000000000000000e+000, 15,
     3.342112340000000e+000,	3.000000000000000e+000, 15,
     1.331242300000000e+000,	1.000000000000000e+000, 15,
     7.783123483200000e-001,	0.000000000000000e+000, 15,
     1.772312223120000e+000,	1.000000000000000e+000, 15,
     2.342340032000000e-003,	0.000000000000000e+000, 15,
     1.231887900000000e-006,	0.000000000000000e+000, 15,
    -1.000000000000000e+000,   -1.000000000000000e+000, 15,
    -9.800000000000000e-001,   -1.000000000000000e+000, 15,
    -8.000000000000000e-001,   -1.000000000000000e+000, 15,
    -8.323231000000000e-001,   -1.000000000000000e+000, 15,
    -5.000000000000000e-001,   -1.000000000000000e+000, 15,
    -2.000000000000000e-001,   -1.000000000000000e+000, 15,
    -1.000000000000000e-005,   -1.000000000000000e+000, 15,
     0.000000000000000e+000,	0.000000000000000e+000, 15,
     2.000000000000000e-001,	0.000000000000000e+000, 15,
     5.503000000000000e-001,	0.000000000000000e+000, 15,
     7.754600000000000e-001,	0.000000000000000e+000, 15,
     3.771102182312300e-001,	0.000000000000000e+000, 15,
     9.998000000000000e-001,	0.000000000000000e+000, 15,
     1.000000000000000e+000,	1.000000000000000e+000, 15,
    -2.000000000000000e+001,   -2.000000000000000e+001, 15,
     6.000000000000000e+001,	6.000000000000000e+001, 15,
     /* Make sure they don't cheat by using ints to compute the floor. */
     /* Floor must work on the range supported by double. */
     4294967297.3,4294967297.0,15,
};
static tv fabs_vector = {
    -2.033232320000000e+001,	2.033232320000000e+001, 15,
    -9.123120000000000e+000,	9.123120000000000e+000, 15,
    -3.342112340000000e+000,	3.342112340000000e+000, 15,
    -1.331242300000000e+000,	1.331242300000000e+000, 15,
    -7.783123483200000e-001,	7.783123483200000e-001, 15,
    -2.342340032000000e-003,	2.342340032000000e-003, 15,
    -1.231887900000000e-006,	1.231887900000000e-006, 15,
     0.000000000000000e+000,	0.000000000000000e+000, 15,
     7.712838210000000e-001,	7.712838210000000e-001, 15,
     2.033232320000000e+001,	2.033232320000000e+001, 15,
     9.123120000000000e+000,	9.123120000000000e+000, 15,
     3.342112340000000e+000,	3.342112340000000e+000, 15,
     1.331242300000000e+000,	1.331242300000000e+000, 15,
     7.783123483200000e-001,	7.783123483200000e-001, 15,
     1.772312223120000e+000,	1.772312223120000e+000, 15,
     2.342340032000000e-003,	2.342340032000000e-003, 15,
     1.231887900000000e-006,	1.231887900000000e-006, 15,
    -1.000000000000000e+000,	1.000000000000000e+000, 15,
    -9.800000000000000e-001,	9.800000000000000e-001, 15,
    -8.000000000000000e-001,	8.000000000000000e-001, 15,
    -8.323231000000000e-001,	8.323231000000000e-001, 15,
    -5.000000000000000e-001,	5.000000000000000e-001, 15,
    -2.000000000000000e-001,	2.000000000000000e-001, 15,
    -1.000000000000000e-005,	1.000000000000000e-005, 15,
     0.000000000000000e+000,	0.000000000000000e+000, 15,
     2.000000000000000e-001,	2.000000000000000e-001, 15,
     5.503000000000000e-001,	5.503000000000000e-001, 15,
     7.754600000000000e-001,	7.754600000000000e-001, 15,
     3.771102182312300e-001,	3.771102182312300e-001, 15,
     9.998000000000000e-001,	9.998000000000000e-001, 15,
     1.000000000000000e+000,	1.000000000000000e+000, 15,
    -2.000000000000000e+001,	2.000000000000000e+001, 15,
     6.000000000000000e+001,	6.000000000000000e+001, 15,
};
static tv ceil_vector = {
    -2.033232320000000e+001,   -2.000000000000000e+001, 15,
    -9.123120000000000e+000,   -9.000000000000000e+000, 15,
    -3.342112340000000e+000,   -3.000000000000000e+000, 15,
    -1.331242300000000e+000,   -1.000000000000000e+000, 15,
    -7.783123483200000e-001,	0.000000000000000e+000, 15,
    -2.342340032000000e-003,	0.000000000000000e+000, 15,
    -1.231887900000000e-006,	0.000000000000000e+000, 15,
     0.000000000000000e+000,	0.000000000000000e+000, 15,
     7.712838210000000e-001,	1.000000000000000e+000, 15,
     2.033232320000000e+001,	2.100000000000000e+001, 15,
     9.123120000000000e+000,	1.000000000000000e+001, 15,
     3.342112340000000e+000,	4.000000000000000e+000, 15,
     1.331242300000000e+000,	2.000000000000000e+000, 15,
     7.783123483200000e-001,	1.000000000000000e+000, 15,
     1.772312223120000e+000,	2.000000000000000e+000, 15,
     2.342340032000000e-003,	1.000000000000000e+000, 15,
     1.231887900000000e-006,	1.000000000000000e+000, 15,
    -1.000000000000000e+000,   -1.000000000000000e+000, 15,
    -9.800000000000000e-001,	0.000000000000000e+000, 15,
    -8.000000000000000e-001,	0.000000000000000e+000, 15,
    -8.323231000000000e-001,	0.000000000000000e+000, 15,
    -5.000000000000000e-001,	0.000000000000000e+000, 15,
    -2.000000000000000e-001,	0.000000000000000e+000, 15,
    -1.000000000000000e-005,	0.000000000000000e+000, 15,
     0.000000000000000e+000,	0.000000000000000e+000, 15,
     2.000000000000000e-001,	1.000000000000000e+000, 15,
     5.503000000000000e-001,	1.000000000000000e+000, 15,
     7.754600000000000e-001,	1.000000000000000e+000, 15,
     3.771102182312300e-001,	1.000000000000000e+000, 15,
     9.998000000000000e-001,	1.000000000000000e+000, 15,
     1.000000000000000e+000,	1.000000000000000e+000, 15,
    -2.000000000000000e+001,   -2.000000000000000e+001, 15,
     6.000000000000000e+001,	6.000000000000000e+001, 15,
};
static tv log10_vector = {
     7.712838210000000e-001,   -1.127857786129964e-001, 15,
     2.033232320000000e+001,	1.308187004574879e+000, 15,
#if 0
     7.000000000000000e+200,	2.008450980400142e+002, 15,
#endif
     9.123120000000000e+000,	9.601433873481835e-001, 15,
     3.342112340000000e+000,	5.240210439481894e-001, 15,
     1.331242300000000e+000,	1.242571087999534e-001, 15,
     1.002121212000000e+004,	4.000920254983285e+000, 15,
     7.783123483200000e-001,   -1.088460791944325e-001, 15,
     1.772312223120000e+000,	2.485402326941420e-001, 15,
     2.342340032000000e-003,   -2.630350059177402e+000, 15,
     1.231887900000000e-006,   -5.909428810537226e+000, 15,
     2.000000000000000e-001,   -6.989700043360187e-001, 15,
     5.503000000000000e-001,   -2.594004871888435e-001, 15,
     7.754600000000000e-001,   -1.104405991750783e-001, 15,
     3.771102182312300e-001,   -4.235316997309089e-001, 15,
     9.998000000000000e-001,   -8.686758342857122e-005, 15,
     1.000000000000000e+000,	0.000000000000000e+000, 15,
     7.732232000000000e+006,	6.888304876242215e+000, 15,
     6.000000000000000e+001,	1.778151250383644e+000, 15,
};
static tv log_vector = {
     7.712838210000000e-001,   -2.596988525360120e-001, 15,
     2.033232320000000e+001,	3.012211895582650e+000, 15,
#if 0
     7.000000000000000e+200,	4.624629287478645e+002, 15,
#endif
     9.123120000000000e+000,	2.210811850844735e+000, 15,
     3.342112340000000e+000,	1.206603044210279e+000, 15,
     1.331242300000000e+000,	2.861125664213119e-001, 15,
     1.002121212000000e+004,	9.212459337382448e+000, 15,
     7.783123483200000e-001,   -2.506273593839496e-001, 15,
     1.772312223120000e+000,	5.722850348108026e-001, 15,
     2.342340032000000e-003,   -6.056604835617891e+000, 15,
     1.231887900000000e-006,   -1.360696268725255e+001, 15,
     2.000000000000000e-001,   -1.609437912434100e+000, 15,
     5.503000000000000e-001,   -5.972916949164239e-001, 15,
     7.754600000000000e-001,   -2.542988773218658e-001, 15,
     3.771102182312300e-001,   -9.752177782108211e-001, 15,
     9.998000000000000e-001,   -2.000200026670447e-004, 15,
     1.000000000000000e+000,	0.000000000000000e+000, 15,
     7.732232000000000e+006,	1.586090812403352e+001, 15,
     6.000000000000000e+001,	4.094344562222100e+000, 15,
};
static tv exp_vector = {
    -2.033232320000000e+001,	1.478373706950614e-009, 15,
    -9.123120000000000e+000,	1.091137109266393e-004, 15,
    -3.342112340000000e+000,	3.536218182219072e-002, 15,
    -1.331242300000000e+000,	2.641489051987330e-001, 15,
    -7.783123483200000e-001,	4.591802941564282e-001, 15,
    -2.342340032000000e-003,	9.976604011057695e-001, 15,
    -1.231887900000000e-006,	9.999987681128588e-001, 15,
     0.000000000000000e+000,	1.000000000000000e+000, 15,
     1.000000000000000e-030,	1.000000000000000e+000, 15,
     7.712838210000000e-001,	2.162540787677973e+000, 15,
     2.033232320000000e+001,	6.764189563832698e+008, 15,
     9.123120000000000e+000,	9.164751079470965e+003, 15,
     3.342112340000000e+000,	2.827879809645889e+001, 15,
     1.331242300000000e+000,	3.785743496637429e+000, 15,
     7.783123483200000e-001,	2.177793805017537e+000, 15,
     1.772312223120000e+000,	5.884443790255884e+000, 15,
     2.342340032000000e-003,	1.002345085453564e+000, 15,
     1.231887900000000e-006,	1.000001231888659e+000, 15,
    -1.000000000000000e+000,	3.678794411714423e-001, 15,
    -9.800000000000000e-001,	3.753110988513996e-001, 15,
    -8.000000000000000e-001,	4.493289641172216e-001, 15,
    -8.323231000000000e-001,	4.350374759478658e-001, 15,
    -5.000000000000000e-001,	6.065306597126334e-001, 15,
    -2.000000000000000e-001,	8.187307530779818e-001, 15,
     0.000000000000000e+000,	1.000000000000000e+000, 15,
     2.000000000000000e-001,	1.221402758160170e+000, 15,
     5.503000000000000e-001,	1.733773071776942e+000, 15,
     7.754600000000000e-001,	2.171590829245811e+000, 15,
     3.771102182312300e-001,	1.458065005861186e+000, 15,
     9.998000000000000e-001,	2.717738226455366e+000, 15,
     1.000000000000000e+000,	2.718281828459045e+000, 15,
    -2.000000000000000e+001,	2.061153622438558e-009, 15,
     6.000000000000000e+001,	1.142007389815684e+026, 15,
};
static tv2 atan2_vector = {
    -2.033232320000000e+001,   -1.478373706950614e-009,  -1.570796326867607e+000, 15,
    -9.123120000000000e+000,	1.091137109266393e-004,  -1.570784366663832e+000, 15,
    -3.342112340000000e+000,	3.536218182219072e-002,  -1.560215933714556e+000, 15,
    -1.331242300000000e+000,   -2.641489051987330e-001,  -1.766674947448043e+000, 15,
    -7.783123483200000e-001,	4.591802941564282e-001,  -1.037785082320439e+000, 15,
    -2.342340032000000e-003,   -9.976604011057695e-001,  -3.139244824884245e+000, 15,
    -1.231887900000000e-006,	9.999987681128588e-001,  -1.231889417548110e-006, 15,
     0.000000000000000e+000,	1.000000000000000e+000,   0.000000000000000e+000, 15,
     1.000000000000000e-030,	1.000000000000000e+000,   1.000000000000000e-030, 15,
     7.712838210000000e-001,   -2.162540787677973e+000,   2.799000286113361e+000, 15,
     2.033232320000000e+001,	6.764189563832698e+008,   3.005877202010196e-008, 15,
     9.123120000000000e+000,	9.164751079470965e+003,   9.954571496178218e-004, 15,
     3.342112340000000e+000,   -2.827879809645889e+001,   3.023953952203074e+000, 15,
     1.331242300000000e+000,	3.785743496637429e+000,   3.381406103688916e-001, 15,
     7.783123483200000e-001,	2.177793805017537e+000,   3.432392896328789e-001, 15,
     1.772312223120000e+000,	5.884443790255884e+000,   2.925445335322908e-001, 15,
     2.342340032000000e-003,	1.002345085453564e+000,   2.336855642068051e-003, 15,
     1.231887900000000e-006,   -1.000001231888659e+000,   3.141591421703411e+000, 15,
    -1.000000000000000e+000,	3.678794411714423e-001,  -1.218282905017278e+000, 15,
    -9.800000000000000e-001,	3.753110988513996e-001,  -1.205056189211196e+000, 15,
    -8.000000000000000e-001,	4.493289641172216e-001,  -1.059044280265646e+000, 15,
    -8.323231000000000e-001,	4.350374759478658e-001,  -1.089170874239997e+000, 15,
    -5.000000000000000e-001,	6.065306597126334e-001,  -6.894194926466748e-001, 15,
    -2.000000000000000e-001,   -1.873075307798180e-002,  -1.664177710933388e+000, 15,
     0.000000000000000e+000,	1.000000000000000e+000,   0.000000000000000e+000, 15,
     2.000000000000000e-001,	1.221402758160170e+000,   1.623057554853668e-001, 15,
     5.503000000000000e-001,	1.733773071776942e+000,   3.073429007746352e-001, 15,
     7.754600000000000e-001,	2.171590829245811e+000,   3.429797699087344e-001, 15,
     3.771102182312300e-001,	1.458065005861186e+000,   2.530913637583888e-001, 15,
     9.998000000000000e-001,	2.717738226455366e+000,   3.525134152962121e-001, 15,
     1.000000000000000e+000,	2.718281828459045e+000,   3.525134217776190e-001, 15,
    -2.000000000000000e+001,	2.061153622438558e-009,  -1.570796326691839e+000, 15,
     6.000000000000000e+001,   -1.142007389815684e+026,   3.141592653589793e+000, 15,
};
/* -1 to a huge power is 1, since such huge power must be even. */
/* Be sure to test -1 to a power not representable in a 32-bit integer. */
static tv2 pow_vector = {
#if 0
    -1.000000000000000e+000,	1.000000000000000e+100,    1.000000000000000e+00, 15,
#endif
    -1.000000000000000e+000,	1.717986918500000e+010,   -1.000000000000000e+00, 15,
    -2.033232320000000e+001,   -1.000000000000000e+000,  -4.918277120442389e-002, 15,
     9.123120000000000e+000,	1.091137109266393e-004,   1.000241258983475e+000, 15,
    -3.342112340000000e+000,	3.000000000000000e+000,  -3.733044197877958e+001, 15,
     1.331242300000000e+000,   -2.641489051987330e-001,   9.272089621478585e-001, 15,
    -7.783123483200000e-001,   -4.000000000000000e+000,   2.725111752962557e+000, 15,
     2.342340032000000e-003,   -9.976604011057695e-001,   4.209166401327696e+002, 15,
     1.231887900000000e-006,	9.999987681128588e-001,   1.231908549376611e-006, 15,
     0.000000000000000e+000,	1.000000000000000e+000,   0.000000000000000e+000, 15,
     1.000000000000000e-030,	1.000000000000000e+000,   1.000000000000000e-030, 15,
     7.712838210000000e-001,   -2.162540787677973e+000,   1.753492232931382e+000, 15,
#if 0
     2.033232320000000e+001,	6.764189563832699e+001,   3.077859839184954e+088, 15,
     9.123120000000000e+000,	9.164751079470965e+001,   9.879875148445376e+087, 15,
#endif
     3.342112340000000e+000,   -2.827879809645889e+001,   1.518150055859838e-015, 15,
     1.331242300000000e+000,	3.785743496637429e+000,   2.953966335009224e+000, 15,
     7.783123483200000e-001,	2.177793805017537e+000,   5.793695724564992e-001, 15,
     1.772312223120000e+000,	5.884443790255884e+000,   2.900821655528386e+001, 15,
     2.342340032000000e-003,	1.002345085453564e+000,   2.309306325547481e-003, 15,
     1.231887900000000e-006,   -1.000001231888659e+000,   8.117757812244982e+005, 15,
     1.000000000000000e+000,	3.678794411714423e-001,   1.000000000000000e+000, 15,
     9.800000000000000e-001,	3.753110988513996e-001,   9.924463728405019e-001, 15,
     8.000000000000000e-001,	4.493289641172216e-001,   9.045977938433523e-001, 15,
     8.323231000000000e-001,	4.350374759478658e-001,   9.232599787342793e-001, 15,
    -5.000000000000000e-001,   -7.000000000000000e+000,  -1.280000000000000e+002, 15,
     2.000000000000000e-001,   -1.873075307798180e-002,   1.030604974945103e+000, 15,
     2.000000000000000e-001,	1.221402758160170e+000,   1.400477412836163e-001, 15,
     5.503000000000000e-001,	1.733773071776942e+000,   3.550245785177498e-001, 15,
     7.754600000000000e-001,	2.171590829245811e+000,   5.756628555494399e-001, 15,
     3.771102182312300e-001,	1.458065005861186e+000,   2.412477378721165e-001, 15,
     9.998000000000000e-001,	2.717738226455366e+000,   9.994565457174982e-001, 15,
     1.000000000000000e+000,	2.718281828459045e+000,   1.000000000000000e+000, 15,
    -2.000000000000000e+001,	2.000000000000000e+000,   4.000000000000000e+002, 15,
     6.000000000000000e+001,   -1.142007389815684e+026,   0.000000000000000e+000, 15,
};
static tv2 fmod_vector = {
    -2.000000000000000e-001, -1.873075307798180e-002, -1.269246922018200e-002, 7,
    -2.033232320000000e+001, -1.478373706950614e-009, -1.169897654446084e-009, 9,
    -9.123120000000000e+000,  1.091137109266393e-004, -1.351571276117514e-005, 12,
    -1.121700000000000e-001,  0.000000000000000e+000,  0.000000000000000e+000, 15,
    -3.342112340000000e+000,  3.536218182219072e-002, -1.806724871407243e-002, 15,
    -1.331242300000000e+000, -2.641489051987330e-001, -1.049777400633489e-002, 15,
    -7.783123483200000e-001,  4.591802941564282e-001, -3.191320541635718e-001, 15,
    -2.342340032000000e-003, -9.976604011057695e-001, -2.342340032000000e-003, 15,
    -1.231887900000000e-006,  9.999987681128588e-001, -1.231887900000000e-006, 15,
     0.000000000000000e+000,  1.000000000000000e+000,  0.000000000000000e+000, 15,
     1.000000000000000e-030,  1.000000000000000e+000,  1.000000000000000e-030, 15,
     7.712838210000000e-001, -2.162540787677973e+000,  7.712838210000000e-001, 15,
     2.033232320000000e+001,  6.764189563832698e+008,  2.033232320000000e+001, 15,
     9.123120000000000e+000,  9.164751079470965e+003,  9.123120000000000e+000, 15,
     3.342112340000000e+000, -2.827879809645889e+001,  3.342112340000000e+000, 15,
     1.331242300000000e+000,  3.785743496637429e+000,  1.331242300000000e+000, 15,
     7.783123483200000e-001,  2.177793805017537e+000,  7.783123483200000e-001, 15,
     1.772312223120000e+000,  5.884443790255884e+000,  1.772312223120000e+000, 15,
     2.342340032000000e-003,  1.002345085453564e+000,  2.342340032000000e-003, 15,
     1.231887900000000e-006, -1.000001231888659e+000,  1.231887900000000e-006, 15,
    -1.000000000000000e+000,  3.678794411714423e-001, -2.642411176571154e-001, 15,
    -9.800000000000000e-001,  3.753110988513996e-001, -2.293778022972007e-001, 15,
    -8.000000000000000e-001,  4.493289641172216e-001, -3.506710358827784e-001, 15,
    -8.323231000000000e-001,  4.350374759478658e-001, -3.972856240521342e-001, 15,
    -5.000000000000000e-001,  6.065306597126334e-001, -5.000000000000000e-001, 15,
     0.000000000000000e+000,  1.000000000000000e+000,  0.000000000000000e+000, 15,
     2.000000000000000e-001,  1.221402758160170e+000,  2.000000000000000e-001, 15,
     5.503000000000000e-001,  1.733773071776942e+000,  5.503000000000000e-001, 15,
     7.754600000000000e-001,  2.171590829245811e+000,  7.754600000000000e-001, 15,
     3.771102182312300e-001,  1.458065005861186e+000,  3.771102182312300e-001, 15,
     9.998000000000000e-001,  2.717738226455366e+000,  9.998000000000000e-001, 15,
     1.000000000000000e+000,  2.718281828459045e+000,  1.000000000000000e+000, 15,
    -2.000000000000000e+001,  2.061153622438558e-009, -4.035849310995987e-010, 15,
     6.000000000000000e+001, -1.142007389815684e+026,  6.000000000000000e+001, 15,
};

static tv3 ldexp_vector = {
     2.772300000000000e+000,	  0,   2.772300000000000e+000, 15,
     3.342112340000000e+000,	 -3,   4.177640425000000e-001, 15,
     1.331242300000000e+000,	 22,   5.583634903859200e+006, 15,
     7.783123483200000e-001,	 70,   9.188690367274831e+020, 15,
     1.772312223120000e+000,	 -5,   5.538475697250000e-002, 15,
     2.342340032000000e-003,	 16,   1.535075963371520e+002, 15,
     1.231887900000000e-006,	  2,   4.927551600000000e-006, 15,
    -1.000000000000000e+000,	  0,  -1.000000000000000e+000, 15,
    -9.800000000000000e-001,	  1,  -1.960000000000000e+000, 15,
    -8.000000000000000e-001,	  3,  -6.400000000000000e+000, 15,
    -8.323231000000000e-001,	  5,  -2.663433920000000e+001, 15,
    -5.000000000000000e-001,	  7,  -6.400000000000000e+001, 15,
    -2.000000000000000e-001,	  9,  -1.024000000000000e+002, 15,
     0.000000000000000e+000,	-11,   0.000000000000000e+000, 15,
};

static tv4 frexp_vector = {
     2.772300000000000e+000,	6.930750000000000e-001,     2, 15,
     3.342112340000000e+000,	8.355280850000000e-001,     2, 15,
     1.331242300000000e+000,	6.656211500000000e-001,     1, 15,
     7.783123483200000e-001,	7.783123483200000e-001,     0, 15,
     1.772312223120000e+000,	8.861561115600000e-001,     1, 15,
     2.342340032000000e-003,	5.996390481920000e-001,    -8, 15,
     1.231887900000000e-006,	6.458640433152000e-001,   -19, 15,
    -1.000000000000000e+000,   -5.000000000000000e-001,     1, 15,
    -9.800000000000000e-001,   -9.800000000000000e-001,     0, 15,
    -8.000000000000000e-001,   -8.000000000000000e-001,     0, 15,
    -8.323231000000000e-001,   -8.323231000000000e-001,     0, 15,
    -5.000000000000000e-001,   -5.000000000000000e-001,     0, 15,
    -2.000000000000000e-001,   -8.000000000000000e-001,    -2, 15,
     0.000000000000000e+000,	0.000000000000000e+000,     0, 15,
};

static tv5 modf_vector = {
#if 0
     2.772300000000000e+100,	0.000000000000000e+000,   2.772300000000000e+100, 15,
#endif
     3.342112340000000e+020,	0.000000000000000e+000,   3.342112340000000e+020, 15,
     1.331242300000030e+002,	1.242300000029957e-001,   1.330000000000000e+002, 15,
     7.783123483200000e-001,	7.783123483200000e-001,   0.000000000000000e+000, 15,
     1.772312223120023e+000,	7.723122231200230e-001,   1.000000000000000e+000, 15,
     2.342340032000000e-003,	2.342340032000000e-003,   0.000000000000000e+000, 15,
     1.231887900312430e-006,	1.231887900312430e-006,   0.000000000000000e+000, 15,
    -1.000000000000000e+000,	0.000000000000000e+000,  -1.000000000000000e+000, 15,
    -9.800000000000001e-001,   -9.800000000000001e-001,   0.000000000000000e+000, 15,
    -8.000000000000010e-011,   -8.000000000000010e-011,   0.000000000000000e+000, 15,
    -8.323231000000100e-001,   -8.323231000000100e-001,   0.000000000000000e+000, 15,
    -5.000000000000000e-001,   -5.000000000000000e-001,   0.000000000000000e+000, 15,
    -2.000000000000000e-001,   -2.000000000000000e-001,   0.000000000000000e+000, 15,
     0.000000000000000e+000,	0.000000000000000e+000,   0.000000000000000e+000, 15,
};

pipepipe main (argc,argv)
   int argc; char *argv[]; {
   int i;
   printf("START OF TEST, ftest\n");
#ifdef HIGHC
#define PI 3.14159_26535_89793_23846_26433_83279_50288_41971_69399_37510
#else
#define PI 3.14159265358979323846264338327950288419716939937510
#endif
#define MAX_EPS 15
#define Count(vector) (sizeof(vector)/sizeof(struct test))
#define Count2(vector) (sizeof(vector)/sizeof(struct test2))
#define Count3(vector) (sizeof(vector)/sizeof(struct test3))
#define Count4(vector) (sizeof(vector)/sizeof(struct test4))
#define Count5(vector) (sizeof(vector)/sizeof(struct test5))

   debug = pv = False;

#  ifdef _NAM
      debug = True;
#  else
      for (i = 0; i < argc; i++) {
         if (strcmp(argv[i],"-d") == 0) debug = True;
         if (strcmp(argv[i],"-pv") == 0) pv = True;
         }
#  endif

   NP10[0] = 1;
   for (i = 1; i < sizeof(NP10)/sizeof(double); i++)
      NP10[i] = NP10[i-1]/10;
   _wait;
   test("acos",acos,acos_vector,Count(acos_vector),False);
   _wait;
   test("asin",asin,asin_vector,Count(asin_vector),False);
   _wait;
   test("atan",atan,atan_vector,Count(atan_vector),False);
   _wait;
   test("tan",tan,atan_vector,Count(atan_vector),True);
   _wait;
   test("sin",sin,asin_vector,Count(asin_vector),True);
   _wait;
   test("cos",cos,acos_vector,Count(acos_vector),True);
   _wait;
   test("cosh",cosh,cosh_vector,Count(cosh_vector),False);
   _wait;
   test("sinh",sinh,sinh_vector,Count(sinh_vector),False);
   _wait;
   test("tanh",tanh,tanh_vector,Count(tanh_vector),False);
   _wait;
   test("sqrt",sqrt,sqrt_vector,Count(sqrt_vector),False);
   _wait;
   test("floor",floor,floor_vector,Count(floor_vector),False);
   _wait;
   test("fabs",fabs,fabs_vector,Count(fabs_vector),False);
   _wait;
   test("ceil",ceil,ceil_vector,Count(ceil_vector),False);
   _wait;
   test("log10",log10,log10_vector,Count(log10_vector),False);
   _wait;
   test("log",log,log_vector,Count(log_vector),False);
   _wait;
   test("exp",exp,exp_vector,Count(exp_vector),False);
   _wait;
   test2("atan2",atan2,atan2_vector,Count2(atan2_vector));
   _wait;
   test2("pow",pow,pow_vector,Count2(pow_vector));
   _wait;
#ifndef NOFMOD
   test2("fmod",fmod,fmod_vector,Count2(fmod_vector));
   _wait;
#endif
   test3("ldexp",ldexp,ldexp_vector,Count3(ldexp_vector));
   _wait;
   test4("frexp",frexp,frexp_vector,Count4(frexp_vector));
   _wait;
   test5("modf",modf,modf_vector,Count5(modf_vector));
   printf("END OF TEST, ftest\n");
   exit(0);
   }
