#if 0
***************************************************************************
*  Copyright 1985-1992 MetaWare Incorporated.  All rights reserved.       *
*  This is an unpublished work, Copyright 1985-1992 by MetaWare           *
*  Incorporated.  Use, disclosure, or transfer without the express        *
*  written permission of MetaWare Incorporated is strictly prohibited.    *
*  If all or part of this source code is incorporated into other source   *
*  code not authored by MetaWare, this copyright notice must              *
*  accompany it.                                                          *
***************************************************************************
*  MetaWare ANSI C Validation Suite                                       *
*  Version 8.92                                                           *
***************************************************************************

   This product comes with no warranty or other guarantees,
   except that if the diskette proves unreadable, MetaWare
   will replace it on an exchange basis within the first 30 days.

   Your license is for internal use only, with no distribution rights,
   at one plant site at a time only.  The plant site may be changed
   without notifying MetaWare.

   The C source code on this diskette is copyrighted by MetaWare
   and is made available for the purpose of testing C compilers and
   other C processors only.  MetaWare accepts no responsibility
   or liability for any kind of damages that result from using the Suite.

   This license is for the use of the software only.  MetaWare
   remains the owner of the software.

************************************************************************
   This C validation suite is contains tests that conform to the ANSI
   Standard for C (X3J11/88-159, Dec. 7 1988).  The suite used to
   attempt to test compilers that were only K&R; this included Lattice
   and Microsoft C for the IBM PC.  But the number of tests and boundary
   cases particular to ANSI increased dramatically and the complexity of
   supporting both ANSI and non-ANSI tests was too high, so the suite is
   only an ANSI suite now.

   It tests required features rather than extensions (extensions are the
   attribution of semantics to a syntactically or constraint-wise
   invalid construct).

   Operation of the suite:

   In which.h you can set STANDALONE if you are testing with no real
   library. This is appropriate for first bringing up a compiler, before
   it is stable enough to compile the library.

   Then, compile suite0.c, suite1.c, suite2.c, suite3.c, and suite4.c.
   suite0.c is the only one that is not a stand-alone program.
   Link:
        suite1 and suite0
        suite2 and suite0
        suite3 and suite0
        suite4 and suite0

   This forms three separate programs.
   The output of the first should be something like this:

Largest unsigned int = 65535 largest signed int= 32767
Largest unsigned short = 65535 largest signed short= 32767
Largest unsigned long = 4294967295 largest signed long= 2147483647
Test preprocessor.
Test #   1 (            f) i1=       a i2=       a OK
Test #   2 (   int/struct) i1=       a i2=       a OK
Test #   3 (        redef) i1=       3 i2=       3 OK
Test #   4 (      noparms) i1=       3 i2=       3 OK
 ... etc, for many hundreds of tests, ending in ...
**** END OF TESTS, suite1 ****

   The result is similar for the other suites.

   If you have formally licensed this program from MetaWare, it comes
   supplied with the output from the MetaWare High C compiler compilation
   and the run.  The High C compiler was run in ANSI mode and thus the
   tests that are specific to MetaWare extensions are not tested.

   If the tests detect an error, "fail" replaces "OK".

   There are a few hundred boundary inequality tests that test optimization
   on code generated for comparisons to 0, -1, and -1.  These tests
   do not print out a "Test #" line unless the test fails.

************************************************************************
#endif

/* Which compiler are we testing? */

#include "which.h"
#include <stddef.h>

#define DEFINING        /* We are defining the globals. */
#include "globals.h"

#ifdef STANDALONE
double fabs(double a) { return a < 0 ? -a : a;}

strlen(char *s) {
   int len = 0;
   while (*s++) len++;
   return len;
   }
/* You must supply a "WMSG" routine that prints out a message to standard output. */
WMSG(char * s) {write(1,s,strlen(s));}

strcmp(const char *s1,const char *s2) {
   int i;
   while (*s1 != 0 && *s2 != 0)
      if (*s1 != *s2) return *s1-*s2;
      else s1++,s2++;
   return 0;
   }

void PW(unsigned long J) {
   void P(unsigned long I) {
      #define BASE 16
      if (I >= BASE) P(I/BASE);
      unsigned int D = I%BASE;
      static char S[2] = {0,0};
      if (D >= 10) S[0] = 'a'+D-10;
      else S[0] = D+'0';
      WMSG(S);
      }
   P(J);
   }

void PD(double D) {
   union u{long lo,hi; double D;};
   PW(((union u*)&D)->hi); WMSG(" "); PW(((union u*)&D)->lo);
   }

EOL() { WMSG("\r\n"); } /* System-dependent. */

announce(s) char *s; { WMSG(s); EOL(); }

#else
announce(s) char *s; { printf("%s\n",s); }
#endif

static uchar uc(i) int i; {return i; }

void Initialize() {
   /* Conventions:
      Arithmetic types:  The first is positive, the second negative (or its
      equivalent in unsigned), the third positive, and the fourth 0.
      Functions:  point to functions 1..4.
   */
   f1 = 1.5; f2 = -2.5; f3 = 3.5; f4 = 0;
   d1 = 1.5; d2 = -2.5; d3 = 3.5; d4 = 0;
   fli1 = Pos; fli2 = Neg; fli3 = Pos2; fli4 = 0;
   uc1 = Pos; uc2 = uc(Neg); uc3 = Pos2; uc4 = 0;
    c1 = Pos;  c2 =    Neg ;  c3 = Pos2;  c4 = 0;
   ui1 = Pos; ui2 = Neg; ui3 = Pos2; ui4 = 0;
   i1 = Pos; i2 = Neg; i3 = Pos2; i4 = 0;
   s1 = Pos; s2 = Neg; s3 = Pos2; s4 = 0;
   us1 = Pos; us2 = Neg; us3 = Pos2; us4 = 0;
   l1 = Pos; l2 = Neg; l3 = Pos2; l4 = 0;
   ul1 = Pos; ul2 = Neg; ul3 = Pos2; ul4 = 0;
   str1 = str2 = str3 = str4;
   str1.x = Pos; str2.x = Neg; str3.x = Pos2; str4.x = 0;
   /* Make sure the other fields aren't equal. */
   str1.y = (str2.y = (str3.y = (str4.y=Pos)+1)+1+1);
   str1.z = (str2.z = (str3.z = (str4.z=Neg)+1)+1+1);
   p0 = array;
   p1 = array+1;
   p2 = array+2;
   p3 = array+3;
   p4 = 0;
   *p1 = Pos; *p2 = Neg; *p3 = Pos2;
   array[0] = 77;
   Testcount = 0;
   Failures = 0;

   /* WE ASSUME 2's complement arithmetic here. */

   /* Discover largest unsigned char.   */
   Big_uc = -1; Big_c = Big_uc/2;

   /* Discover largest unsigned int.    */
   Big_ui = -1; Big_i = Big_ui/2;

   /* Discover largest unsigned short.  */
   Big_usi = -1; Big_si = Big_usi/2;

   /* Discover largest unsigned int.    */
   Big_uli = -1; Big_li = Big_uli/2;
   }


void Results() {
#ifndef STANDALONE
    printf("%d Tests, %d Failures.\n", Testcount, Failures);
#else
    PW(Testcount); WMSG(" Tests, "); PW(Failures); WMSG(" Failures.");
    EOL();
#endif
    }


/****************************************************/
void equal(int i1, int i2, char *msg, int line) {
   int failed = i1 != i2;
   ++Testcount;
   if (failed) Failures++;
   if (!Silent || failed) {
#ifndef STANDALONE
      printf("Test # %3d L%4d (%26.26s) i1=%8x i2=%8x %s\n",
         Testcount, line, msg, i1, i2, failed ? "fail <--" : "OK");
#else
      WMSG("Test # "); PW(Testcount); WMSG(" L"); PW(line);
      WMSG("("); WMSG(msg); WMSG(") i1="); PW(i1); WMSG(" i2=");
      PW(i2); WMSG(" "); WMSG( failed ? "fail <--" : "OK");
      EOL();
#endif
      }
   }

/****************************************************/
void lequal(long i1, long i2, char *msg, int line) {
   int failed = i1 != i2;
   ++Testcount;
   if (failed) Failures++;
   if (!Silent || failed) {
#ifndef STANDALONE
      printf("Test # %3d L%4d (%26.26s) i1=%8lx i2=%8lx %s\n",
         Testcount, line, msg, i1, i2, failed ? "fail <--" : "OK");
#else
      WMSG("Test # "); PW(Testcount); WMSG(" L"); PW(line);
      WMSG("("); WMSG(msg); WMSG(") i1="); PW(i1); WMSG(" i2=");
      PW(s2); WMSG(" "); WMSG( failed ? "fail <--" : "OK");
      EOL();
#endif
      }
   }

/****************************************************/
void sequal(char * s1, char * s2, char * msg, int line) {
    int failed = strcmp(s1, s2) != 0;
    ++Testcount;
    if (failed) Failures++;
    if (!Silent || failed) {
#ifndef STANDALONE
	printf("Test # %3d L%4d (%26.26s) s1=<%s> s2=<%s> %s\n",
                Testcount, line, msg, s1, s2, failed ? "fail <--" : "OK");
#else
        WMSG("Test # "); PW(Testcount); WMSG(" L"); PW(line);
	WMSG("("); WMSG(msg); WMSG(") s1=<"); WMSG(s1); WMSG("> s2=<");
	WMSG(s2); WMSG("> "); WMSG( failed ? "fail <--" : "OK");
	EOL();
#endif
        }
    }

static int IS_NAN(d)
    double d;
    {
    union {
    long l[sizeof(d)/sizeof(long)];
    double d;
    }u;
#ifdef _I386
    u.d = d;
    if (  ( (u.l[1] & 0x7ff00000) == 0x7ff00000) &&
       /* have a NAN or Infinity */
          ( (u.l[1] & 0x000fffff) || u.l[0] ) )
            /* Not Infinity thus is NAN */
        return 1;
    else
        return 0;
#else
    if (d==0.0){  /* A NAN is considered equal to zero, but not really */
    int i;
    u.d = d;
    for (i=0; i<sizeof(u.l)/sizeof(long);i++)
        if (u.l[i]) return 1;
    return 0;
    }
    if (d != d || d*2.0 == d || d *0.5==d) return 1;
     return 0;
#endif
    }

/**********************************************/
void dequal(double d1, double d2, double Tolerance, char *msg, int line) {
    extern double fabs(); double diff;
    /* Test for nan's: d1==d1 will be false.*/
    int failed = fabs(diff=d1-d2) > Tolerance || IS_NAN(d1) || IS_NAN(d2);
 
    if (failed) Failures++;
    if (!Silent || failed) {
#ifndef STANDALONE
        printf("Test # %3d L%4d (%13.13s) d1=%f d2=%f Tol=%f diff=%e %s\n",
                Testcount, line, msg, d1, d2, Tolerance, diff,
	        failed ? "fail <--" : "OK");
#else
        WMSG("Test # "); PW(Testcount); WMSG(" L"); PW(line);
	WMSG("("); WMSG(msg); WMSG(") d1="); PD(d1); WMSG(" d2="); PD(d2);
	WMSG(" Tol="); PD(Tolerance);
	WMSG( failed ? "fail <--" : "OK");
	EOL();
#endif
       }
   }

/**********************************************/
void equal_struct(s1,s2,msg,line)
   struct s s1,s2; char *msg; int line;
{

   char fail =  s1.x!=s2.x || s1.y != s2.y || s1.z != s2.z;
   ++Testcount;
   if (fail) Failures++;
   if (!Silent || fail) {
#ifndef STANDALONE
      printf("Test # %3d L%4d (%26.26s)               %s\n",
      Testcount, line, msg, fail ? "fail <--" : "OK");
#else
      WMSG("Test "); WMSG(msg); WMSG(" line:"); PW(line);
      WMSG(": "); PW(Testcount);
      WMSG(fail ? "  fail <--" : "  OK");
      EOL();
#endif
    }
}

int int_ident(i) int i; {return i; }
long int longint_ident(li) long int li; {return li; }

/**********************************************/
#if 0
char *i2attmpstr(i)
   int i;
{
   static char i2atmpstrbuf[32];
   sprintf(i2atmpstrbuf, "@_%d", i);
   return i2atmpstrbuf;
   }


char *si2attmpstr(s,i)
   char *s ;int i;
{
   static char si2atmpstrbuf[256];
   sprintf(si2atmpstrbuf, "%s @_%d", s, i);
   return si2atmpstrbuf;
   }

#endif
