#ifndef lint
static const char rcsid[] = "$Header: /sun3/tlib/suite_c/RCS/suite8.c,v 1.10 1994/02/18 22:16:45 pickens Exp $";
#endif

/*
 * $Log: suite8.c,v $
 * Revision 1.10  1994/02/18  22:16:45  pickens
 * Put in "test_switch()"
 *
 * Revision 1.9  1994/01/28  16:00:39  pickens
 * i386 ===> j386.   (i386 is a macro on 386 UNIX)
 *
 * Revision 1.8  1994/01/28  04:38:43  richf
 * added test_many_args()
 *
 * Revision 1.7  1994/01/28  04:27:50  richf
 * Budi's test_6955() and David's loop test.
 *
 * Revision 1.6  1994/01/12  21:37:34  richf
 * added test_csetail_bug()
 *
 * Revision 1.5  1993/12/22  22:03:15  richf
 * added test_CC_optimization()
 *
 * Revision 1.4  1993/12/22  21:59:23  richf
 * Checking in for David: float tests, ssa tests, OEM tests, etc.
 *
 * Revision 1.3  1993/09/08  22:13:52  richf
 * make test_inline() work for C as well as C++
 *
 * Revision 1.2  1993/09/08  17:26:29  miker
 * if MSDOS, test inline intrinsic. CSR 14692
 * makes it clear that we weren't testing this
 * before.
 *
 * Revision 1.1  1993/02/23  19:34:43  pickens
 * Initial revision
 *
 */

#include "which.h"
#include "globals.h"
#include <stdio.h>
#include <string.h>
#include <stddef.h>
#include <stdlib.h>
#include <assert.h>
#include <setjmp.h>
#include <stdarg.h>
#include <errno.h>
#include <math.h>
#include <limits.h>

#ifdef NOLONGDOUBLE
# define LONG_DOUBLE double
#else
# define LONG_DOUBLE long double
#endif

/*
 * The following is an acid 387 tester.
 * Added 2/23/93 by DP
 */

/*****************************************
   RE: atan2() problem

   Jan 25, 1993

   compile with:  hc386 barnes.c -fsoft          (correct results)
   compile with:  hc386 barnes.c [with -387]     (incorrect results)

   The compiler generates incorrect results when compiling on a machine
   WITH A CO-PROCESSOR.  The problem does not occur when compiling with
   -fsoft or with -Hoff=Recognize_library (this toggle turns off inlining).

   The following piece of code contains two assignment statements each for
   LunarLongCorr and TEMP:  one type assigns an constant; the other type
   assigns the result of an equation using combinations of sin, cos, and
   atan2.

   When you comment out either one of the two statements containing an 
   equation, the results of the code are correct.

   When you leave the statements containing the equations uncommented AND
   add a printf() statement AFTER the assignment statements, the results 
   are correct.
*/


/*
** Globals:
**/
                /*******************/
                /****  IMPORTS  ****/
                /*******************/


void test_387()
{
/*
** Locals:
**/
static double        ac_LunarRefVector[3];   /* lunar position vector   */             
static double SMTime;               /* days since 0 hr, 1 Jan 85        */
static double MeanEcliptObliq;      /* mean obliquity of the ecliptic   */
static double LongMeanAscendNode;   /* longitude of mean ascending node of moon  */
static double MeanLongPerigee;      /* mean longitude of lunar perigee  */
static double MeanLongMoon;         /* mean longitude of the moon       */
static double MeanAnomalySun;       /* mean anomaly of the sun          */
static double MeanElongMoon;        /* mean elongation of the moon      */
static double LunarLongCorr;        /* correction to longitude of moon  */
static double RME[3];               /* lunar pos vect, ecliptic coord   */
static double RM[3];                /* lunar pos vect, equatorial coord */
static double LunarParallax;        /* lunar parallax                   */
static double S,AM,TEMP,ALAMM,      /* intermediate variables --        */
   BETAM,COBLIC,SOBLIC,R,AMR,       /*    adopted from SMPOS.FOR        */
   T4R,SR,DMSR,t1,t2,t3,t4;
static double deg2rad = 0.01745329251994330e0;
static double AI = 5.145396374e0;   /* adopted from SMPOS.FOR           */
static int i;                              /* counter                          */
static double LRV[] = {-358221.202166, 151973.116306, 80957.328453 };
static double RManswer[] = {
		-.901282112944163,0.382363328990526,0.203688088827471};
static double RMEanswer[] = {
		-.901282112944163,0.431836380391776,0.0347547616454908};

/****************************************************************************/
/*************************  END OF DECLARATIONS  ****************************/


   SMTime = (double) (10000 / 86400.0) - 5700.;
   MeanEcliptObliq   = (double) (23.441242 - 3.5626e-7 * SMTime);
   LongMeanAscendNode= (double) (55.204723 - 0.05295378 * SMTime);
   MeanLongPerigee   = (double) (192.917585 + 0.11140361 * SMTime);
   MeanLongMoon      = (double) (18.251907 + 13.17639649 * SMTime);
   MeanAnomalySun    = (double) (356.930969 + 0.98560028 * SMTime);
   MeanElongMoon     = (double) (98.640536 + 12.19074913 * SMTime);
   S  = MeanLongMoon - LongMeanAscendNode;
   AM = MeanLongMoon - MeanLongPerigee;

/* *********************************************************************
 * **************  THIS IS THE INTERESTING PART HERE  ******************
 * *********************************************************************
 */
   /* correction to the longitude of the moon */
/**/
   LunarLongCorr = 6.2887 * sin(AM * deg2rad)
      - 1.2740 * sin((AM - 2 * MeanElongMoon) * deg2rad)
      + 0.6583 * sin(2 * MeanElongMoon * deg2rad)
      - 0.1855 * sin(MeanAnomalySun * deg2rad)
      - 0.1143 * sin(2 * S * deg2rad)
      + 0.053 * sin((AM + 2 * MeanElongMoon) * deg2rad)
      - 0.046 * sin((MeanAnomalySun - 2 * MeanElongMoon) * deg2rad)
      - 0.035 * sin(MeanElongMoon * deg2rad)
      + 0.2130 * sin(2 * AM * deg2rad)
      - 0.059 * sin(2 * (AM - MeanElongMoon) * deg2rad)
      - 0.057 * sin((AM + MeanAnomalySun - 2 * MeanElongMoon) * deg2rad)
      + 0.041 * sin((AM - MeanAnomalySun) * deg2rad);

/**/
/* LunarLongCorr = -5.98018117505753600;*/

  TEMP  = atan2( sin( S*deg2rad )*cos( AI*deg2rad ), cos( S*deg2rad ) ) / deg2rad;
/*  TEMP = 157.36411821071908430;*/

/*   printf("howdy howdy\n");*/

/* **********************************************************************/
/* **********************************************************************/

   ALAMM = TEMP + LongMeanAscendNode;
   BETAM = atan(sin(S * deg2rad) * tan(AI * deg2rad)) / deg2rad;
   COBLIC= cos(MeanEcliptObliq * deg2rad);
   SOBLIC= sin(MeanEcliptObliq * deg2rad);

/* compute moon vector */
           
   /* moon vector in ecliptic coordinates */
   RME[0]= cos(ALAMM * deg2rad) * cos(BETAM * deg2rad);
   RME[1]= sin(ALAMM * deg2rad) * cos(BETAM * deg2rad); 
   RME[2]= sin(BETAM * deg2rad);

   /* rotate to equatorial coordinates */ 
   RM[0]= RME[0];
   RM[1]= RME[1] * COBLIC - RME[2] * SOBLIC;
   RM[2]= RME[2] * COBLIC + RME[1] * SOBLIC;

   /* intermediate variables */
   AMR = AM * deg2rad;
   T4R = MeanAnomalySun * deg2rad;
   SR  = S * deg2rad;
   DMSR= MeanElongMoon * deg2rad;

   /* compute lunar parallax */
   LunarParallax= (double)((3422.7 + 186.5398 * cos(AMR)
      + 34.3117 * cos(AMR - 2 * DMSR)
      + 28.2373 * cos(2 * DMSR) + 10.1657 * cos(2 * AMR)
      + 3.0861 * cos(AMR + 2 * DMSR)
      + 1.9178 * cos(T4R - 2 * DMSR)
      + 1.4437 * cos(AMR + T4R - 2 * DMSR)
      + 1.1528 * cos(AMR - T4R)) / 3600);

   /* intermediate variable */
   R= 6378.388 / sin(LunarParallax * deg2rad);

   /* compute moon vector in geocentric 
           inertial coordinates,
      mean equator and equinox of date */

   for(i= 0; i < 3; i++)
   {   ac_LunarRefVector[i] = RM[i] * R;
       dequal(ac_LunarRefVector[i],LRV[i],0.0001,"387");
       dequal(RM[i],RManswer[i],0.000000000001,"RM");
       dequal(RME[i],RMEanswer[i],0.000000000001,"RME");
       }
   dequal(AMR,-1302.77681142296,0.0000001,"AMR");
   dequal(R,397457.352166712,0.0000001,"R");
   dequal(T4R,-91.8196173889615,0.000000001,"T4R");
   dequal(SR,-1316.72382889868,0.000000001,"SR");
   dequal(DMSR,-1211.03542230056,0.000000001,"DMSR");
   dequal(BETAM,1.99170225826609,0.00000000001,"BETAM");
   dequal(ALAMM,514.399258301003,0.0000000001,"ALAMM");
   dequal(LunarParallax,0.91952104726854,0.0000000000001,"LunarPar");
   dequal(LunarLongCorr,-5.98018117505725,0.0000000000005,"LunarLong");
}  /* end of function: AC_CalculateLunarModel */

/***********************  end of file:  aclunar.c  **************************/


/*
 * The following test was reported by NCR. It caused the 2.4 inliner to
 * break.
 */
static int z = 10;
static void ifgoo()
    {
    z++;
    }

void test_if(){
    static int x=0,y;
    y = ((x?(void)0:ifgoo()),z+1);
    equal(y,12,"?:");
    }

void test_cc()	/* Added 3/1/93 by DP*/
{
static int i = 0x7FFFFFFF;
static unsigned u = 0xFFFFFFFF;
if (i+1 > 0)		/* Should be False */
    equal(1,0,"cc");
if (i+1 >= 0)		/* Should be False */
    equal(0,1,"cc");
if (u+1 > 0)		/* Should be false */
    equal(0,1,"cc.u");
}

/*
 * Added 3/8/93: failed in 2.5 inliner
 */
static int inlineable(int i){
    return i+1;
    }
static int iX[10];
static int iXX = 0;
static void test_inline()
{
iX[iXX++] = inlineable(99);
equal(iXX,1,"inline");
equal(iX[0],100,"inline");
}

static void test_floatint()
    {
    #define INT_MIN -2147483648
    #define INT_MAX  2147483647

    float _min = INT_MIN ;
    float _max = INT_MAX ;
    /* INT_MIN is unsigned! */
    dequal(_min,2147483648.0,1.0,"_min");
    dequal(_max,2147483647.0,1.0,"_max");
    }

static void test_fltfold()		/* Added 4/14/93 by JDP*/
    {
#ifndef _IBM370	/* Only works for IEEE floating point*/
    union {
	float f;
	long i;
	} u;
    long l;
    float f;
    u.i = 0x3F800000;
    dequal(u.f,1.0,0.0,"fltfold.1");
    l = 0x3F800000;
    f = *(float*)&l;
    dequal(f,1.0,0.0,"fltfold.2");
#endif
    }


static void test_byte()	/* Added 4/14/93 by JDP */
    {
    unsigned short j,k,l;
    union { short s; char c; }u;
    j = 0xabcd;
    k = ((char *)&j)[0];
    l = ((char *)&j)[1];
    u.s = 1;
    if (u.c==0){
	equal(k,(unsigned short)(char)0xab,"byte.1");
	equal(l,(unsigned short)(char)0xcd,"byte.2");
	}
    else{
	equal(k,(unsigned short)(char)0xcd,"byte.3");
	equal(l,(unsigned short)(char)0xab,"byte.4");
	}
    }


/*** longstr.c ******
clcc -Xa dumps a core when preprocessing this very long string. 
There seems to be a limitation for the argument of the citaton operator. 
Such a limitation seems not to exist when using clcc in K&R mode and 
using macro substitution inside literal strings. 
Leave out the word "very" in the first sentence, and clcc compiles just fine.
*******/
static void test_longstring()  /* Added 4/16/93 by JDP */

{
#define STRING(x) #x
   equal(
       strcmp(
  STRING(
(clcc -Xa dumps a core when preprocessing this very long string. There seems to be a limitation for the argument of the citaton operator. Such a limitation seems not to exist when using clcc in K&R mode and using macro substitution inside literal strings.)),
"(clcc -Xa dumps a core when preprocessing this very long string. There seems to be a limitation for the argument of the citaton operator. Such a limitation seems not to exist when using clcc in K&R mode and using macro substitution inside literal strings.)"
       ),
   0,"longstr");
}

/*
 * The following failed in 2.5 due to inlining bug.
 * Added 4/23/93 by JDP
 */
static int fwd(int *a);
test_fwdinline()
    {
    int a[2000];  /* Prevents this function from being  inlined into main*/
    fwd(a);
    }
static int fi1=1,fi2=2;
static int fwd(int *a)
    {
    equal(fi1,1,"fwdin.1");
    equal(fi2,2,"fwdin.1");
    }


/* test case produced by mats mnhav

   CSR 13296
   RE: asin produces NaN

   compile as follows:
      hc386 t.c -f387
   The program will produce some erronoues float as NaN.
   alfa and x0 produce NaN; x0 depends on alfa;
   alfa uses asin.

   tested with v3.1 and v3.04.
   According to Mats, compiling it with 1.73 instead will work just fine.
*/
#define PI 3.14159265359
static float   flt_d  = 30;
static float   flt_h  = 3.5;
static float   flt_d4 = 28.6;
static float   flt_d2 = 2;
static float   flt_a  = 5;

static void test_387_again()
   {
   float x, y, x0, x1, y1, x2, y2, x3, y3, x4, y4, alfa, beta, gamma,scrap;
   double r2;

   beta=180.0/PI*(float)asin(flt_a/flt_d4/2.0);
   scrap=flt_a-0.7*flt_h;
   x=flt_d4/2.0*(float)cos(5.0*PI/180.0);
   y = -flt_d4/2.0*(float)sin(5.0*PI/180.0);
   alfa=180.0/PI*(float)asin(-y/(flt_d4/2.0+flt_a));
   x0=(flt_d4/2.0+flt_a)*(float)cos(alfa*PI/180.0);
   x1=(flt_d4/2.0+flt_a)*(float)cos((beta*1.6+5.0)*PI/180.0);
   y1 = -(flt_d4/2.0+flt_a)*(float)sin((beta*1.6+5.0)*PI/180.0);
   x2=(flt_d4/2.0+flt_a-0.2*scrap)*(float)cos((beta*2.2+5.0)*PI/180.0);
   y2 = -(flt_d4/2.0+flt_a-0.2*scrap)*(float)sin((beta*2.2+5.0)*PI/180.0);
   x3=(flt_d4/2.0+flt_a-0.8*scrap)*(float)cos((beta*3.5+5.0)*PI/180.0);
   y3 = -(flt_d4/2.0+flt_a-0.8*scrap)*(float)sin((beta*3.5+5.0)*PI/180.0);
   x4=(flt_d4/2.0+0.7*flt_h)*(float)cos((beta*4.4+5.0)*PI/180.0);
   y4 = -(flt_d4/2.0+0.7*flt_h)*(float)sin((beta*4.4+5.0)*PI/180.0);
   scrap = -flt_h-flt_d4/2.0;
   r2 = -scrap+(scrap*scrap-x4*x4-y4*y4)/(2.0*scrap-2.0*x4);
   gamma=180.0/PI*(float)asin(-y4/r2);
#  define X(var,ans) dequal(var,ans,fabs(ans)*5.0E-7,"flt." #var)
   X(x , 14.245584);
   X(y , -1.246327);
   X(x0 , 19.259716);
   X(x1 , 18.803551);
   X(y1 , -4.349313);
   X(x2 , 18.059166);
   X(y2 , -5.189470);
   X(x3 , 15.940193);
   X(y3 , -6.619503);
   X(x4 , 14.915723);
   X(y4 , -7.621267);
   X(alfa , 3.702539);
   X(beta , 5.014773);
   X(gamma , 26.226847);
   X(scrap , -17.799999);
   X(r2 , 17.245565);
#  undef x
   }

/*

	CSR 13257. Bad 387 code generation.

	Added May 4, 1993 by JDP

	This code illustrates the problem.
	If correct, the program should print  "1 0".
	As is, it prints "1 0" when compiled with R1.73, and "1 1" with R3.04.
	With R3.04, the program works correctly if
		a. "long double" in line 50 is changed to "double", or
		b. the call to "dummy" in line 74 is uncommented.

	Looking at the generated code, it appears that the compiler
	loses track of what's where on the 387 stack when compiling
	the return expression.  The call to "dummy" merely forces
	the compiler to store everything, which it does correctly.

*/
	
	


typedef int PIX;
typedef struct _POINT { double x,y,z; } POINT;

int leanleft=0;
POINT p[8]={
	{-2,0,0},{2,0,0},{0,1,0},{0,-1,0},
	{-1,0,0},{1,0,0},{0,2,0},{0,-2,0}};

#define P(i) (p[i])

void dummy() {}

int flp(i0,i1,i2,i3) PIX i0,i1,i2,i3; {
   POINT p0,p1,p2,p3;
   long double d0x,d1x,d2x,ddx,d0y,d1y,d2y,ddy,d0z,d1z,d2z;
   extern int memcmp();
   p0=P(i0);
   p1=P(i1);
   p2=P(i2);
   p3=P(i3);
   if (!memcmp(&p1.x,&p2.x,sizeof(p1.x))
	&& !memcmp(&p0.y,&p2.y,sizeof(p1.x))
	&& !memcmp(&p0.x,&p3.x,sizeof(p1.x))
	&& !memcmp(&p3.y,&p1.y,sizeof(p1.x))) return !leanleft;
   d0x=p3.x-p2.x;
   d0y=p3.y-p2.y;
   d1x=p1.x-p2.x;
   d1y=p1.y-p2.y;
   if ((ddx=d0x*d1y-d1x*d0y)<=0) return 0;
   d2x=p0.x-p2.x;
   d2y=p0.y-p2.y;
   if ((ddy=d2x*d0y-d0x*d2y)<=0) return 0;
   d0z=d0x*d0x+d0y*d0y;
   d1z=d1x*d1x+d1y*d1y;
   d2z=d2x*d2x+d2y*d2y;

/*

	dummy();

*/

   return d0x*((d2x*d0z-d0x*d2z)*ddx - (d0x*d1z-d1x*d0z)*ddy) <
	  d0y*((d2z*d0y-d0z*d2y)*ddx - (d0z*d1y-d1z*d0y)*ddy);
}


static void test_387_yet_again()
    {
    equal(flp(0,1,2,3),1,"387");
    equal(flp(4,5,6,7),0,"387");
    }

struct XX {
    char           c;
    signed         char sc;
    short          s;
    int            i;
    unsigned       char uc;
    unsigned short us;
    unsigned int   ui;
    long           l;
    unsigned long  ul;
    float          f;
    int            g;
    double         d;
    long double    ld;
};

static struct XX x2 = { 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 3, 17, 18, };

struct XX Od() {struct XX x1; x1=x2; return(x1);}
struct XX Old(){struct XX x1; x1=x2; return(x1);}

static void test_amd_structs() {
   double temp_od  = - Od().d ;
   double temp_old = - Old().ld ;
   dequal(temp_od,-17.0,0.0,"amd.1");
   dequal(temp_old,-18.0,0.0,"amd.2");
}

typedef struct { char x ; } S1 ;

S1 RetS1() {
   S1 s1a ;
   s1a.x = (char)100 ;
   return (s1a) ;
}

static void test_amd_structs2() {
   S1 s1a ;
   s1a = RetS1() ;
   equal((char)100, s1a.x, "amd2.1") ;
   equal((char)100, RetS1().x,"amd2.2") ;
   }


struct {
	short a;
	short p1:1,p2:1,p3:1,p4:1,p5:1; 
	int p6:1;
    } sbits = { 0x9999,1,1,1,1,1,1 };
struct {
	short a;
	int p1:1,p2:1,p3:1,p4:1,p5:1; 
	short p6:1;
    } tbits = { 0x9999,1,1,1,1,1,1 };

static void test_bits(void)
    {
    equal(sbits.p6,1,"sbits");
    equal(tbits.p6,1,"tbits");
    }

static void test_wierd_loops()
    {
    int i, k;
    k = 0;
    goto l2;

l1: k++;
    goto l6;

l2: for (i=1; i<=5; i++) {
       k++;
       goto l4;
l3:    if (i == 5) goto l5;
	}
l4: k++;
    goto l3;

l5: for(i=1; i<=5; i++) {
       k++;
       goto l1;
l6:    if (i == 5) goto l7;
	}
l7: equal(k,20,"wierd");
    }


void test_valnum(void)
	{
	auto char Jc[12][12];
	auto signed char Jsc[12][12];
	auto short Js[12][12];
	auto int Ji[12][12];
	auto unsigned char Juc[12][12];
	auto unsigned short Jus[12][12];
	auto unsigned int Jui[12][12];
	auto long Jl[12][12];
	auto unsigned long Jul[12][12];
	int x_8 = 8;
	int y_8 = 8;

	auto float Jf[12][12];
	auto double Jd[12][12];
	auto long double Jld[12][12];
	Jc[0][0] = 7;
	Jsc[1][1] = 8;
	Js[2][2] = 9;
	Ji[3][3] = 10;
	Juc[4][4] = 11;
	Jus[5][5] = 12;
	Jui[6][6] = 13;
	Jl[7][7] = 14;
	Jul[x_8][y_8] = 15;
	equal(Jul[8][8],15,"valnum");
	}

/*
 * test_fltmat - added July 15, 1993. Caused 387 code-generator to loop!
 */
typedef double Float;
typedef struct Vector {
        Float x, y, z;
} Vector;

typedef struct {
        Float x, y, z, theta;
} Rotate;
typedef struct RSMatrix {
        Float matrix[2][3];
} RSMatrix;


/*
 * Initialize a rotation matrix given an axis of rotation and an
 * angle.  Right-handed rotation is applied.
 */
void
RotationMatrix(x, y, z, theta, trans)
Float x, y, z, theta;
RSMatrix *trans;
{
	Float n1, n2, n3, sintheta, costheta;
	Vector vector;
/*	MatrixInit(trans);*/
	vector.x = x;
	vector.y = y;
	vector.z = z;

#if 0
	if (VecNormalize(&vector) == 0.)
		RLerror(1, "Degenerate rotation axis.\n");
#endif

	sintheta = sin(theta);
	costheta = cos(theta);

	n1 = vector.x; n2 = vector.y; n3 = vector.z;
	trans->matrix[0][0] = 0.0;
	trans->matrix[0][1] = (Float)(n1*n2*(1 - costheta) + n3*sintheta); 
	trans->matrix[0][2] = (Float)(n1*n3*(1 - costheta) - n2*sintheta);
	trans->matrix[1][0] = (Float)(n1*n2*(1 - costheta) - n3*sintheta);
	trans->matrix[1][1] = (Float)(n2*n2 + (1 - n2*n2)*costheta);
	trans->matrix[1][2] = (Float)(n2*n3*(1 - costheta) + n1*sintheta);
}

void test_fltmat()
{
int i;
RSMatrix trans;
RSMatrix ans = { 
    {{0.000000000000000e+00, 1.767949192377280e+00, -5.980762113498430e-01},
     {-1.232050807544960e+00, 1.401923788624240e+00, 1.303847577235510e+00}}
    };
RotationMatrix(1.0,2.0,3.0, 3.1415926535/6, &trans);

for(i=0; i<2; i++){
    int j;
    for (j=0; j<3;j++)
	dequal(ans.matrix[i][j],trans.matrix[i][j],1.e-14,"fltmat");
    }
}

/*
 *  Test induction variable analysis
 */

#define INBOUND(A,B,C) ((A) <= (B) && (A) >= (C))
long            for_loops();
int Q1 = 1, Q2 = 2, Q3 = 3, Q4 = 4, Q5 = 5;
int Q6 = 6, Q7 = 7, Q8 = 8, Q9 = 9, Q10 = 10;


void test_induct(void)
{
        int                      k;
        int                      len;
        long int                 i;
        char                     c;
        int                      xtotal = 0;
        
        i = for_loops();
	equal(i,32768,"induct");
        xtotal += Q1 + Q2 + Q3 + Q4 + Q5 + Q6 + Q7 + Q8 + Q9 + Q10;
	equal(xtotal,55,"induct");
}

long 
for_loops(void)
{
   int s, b, c, d, e, f, g, h, i, j, k, l, m, n, o;
   long x = 0;

   for (s = 1; s <= 2; s++)           /* this statement is not nested *//* 0 */
      for (b = 1; b <= 2; b++)                                          /* 1 */
         for (c = 1; c <= 2; c++)                                       /* 2 */
            for (d = 1; d <= 2; d++)                                    /* 3 */
               for (e = 1; e <= 2; e++)                                 /* 4 */
                  for (f = 1; f <= 2; f++)                              /* 5 */
                     for (g = 1; g <= 2; g++)                           /* 6 */
                        for (h = 1; h <= 2; h++)                        /* 7 */
                           for (i = 1; i <= 2; i++)                     /* 8 */
                              for (j = 1; j <= 2; j++)                  /* 9 */
                                 for (k = 1; k <= 2; k++)               /* 10 */                                    for (l = 1; l <= 2; l++)            /* 11 */                                       for (m = 1; m <= 2; m++)         /* 12 */                                          for (n = 1; n <= 2; n++)      /* 13 */                                             for (o = 1; o <= 2; o++)   /* 14 */                                                x++;                    /* 15 */
    return x;
}



/*
 * test for pure 387 compatibility of floating point
 */
void test_float_prec()	/* Added Jul 22, 1993  by JDP*/
{
        double  x, y, fabs(), f();
#if _I386 
	#define result (sizeof(long double) > sizeof(double)?2.0:1.0)
#else
	#define result 1.0
#endif
	

        x = 4503599627370497.0;
        y = 0.5 - 1.0 / 16384.0;
	dequal(f(x+y),result,0.0,"fp.prec");
        dequal(f( fabs( x ) + fabs( y ) ),result,0.0,"fp.prec" );
#undef result
}

double f( x )
double x;
{
    int array[50000]; /* Prevent inlining of this function */
    return x - 4503599627370496.0;
}


void test_atan()	/* added July 23, 1993 by JDP */
{
    double y, x;
    y = 1.55741;    x = 1.0;
    dequal(atan2(-y,x), -1.00000066423303, 1.0e-10, "Test2_atan2.");
    dequal(atan2(y,-x),   2.14159198935676,  1.0e-10, "Test3_atan2.");
    dequal(atan2(-y,-x), -2.14159198935676,  1.0e-10, "Test4_atan2.");
}

void test_inline1() {   /* Added Sept. 8, MLR */
#if _MSDOS
    _inline(0xaf);
#endif
   }

/*
 * Added: 9/28/93 by D.P.
 * Exposed flaw in SSA transformation.
 */
void test_ssa1()
{
int i=0,j;
int k;
for (k=0; k<100; k++){
    if (k==50){
	i = 1;
	j = k;
	}
    }
if (i)
    equal(j,50,"ssa");
}


/* Added Oct 6, 1993 */

/* Tests of ARCSIN, ARCCOS und ARCTAN via Taylorreihe */

static char *t = "%s, arg. %f, tay. %f, val. %f\n";

static double func_3 ();

static double func_1 (x)
double x; {

	double a,b,s;
	int i;

	a = x*x;
	s = 0;
	b = x;
	for (i = 1; fabs(b) > fabs(s) * 1e-16; ) {
		b *= a*i*i/((i+1.)*(i+2.));
		s += b;
		i += 2;
	}
	return s;
}

static double func_2 (x)
double x; {

	if(x >= .8) return 1.5707963267948966192 - func_3 (x);
	if(x < -.8) return func_3 (-x) - 1.5707963267948966192;
	return x + func_1(x);
}

static double func_3 (x)
double x; {

	double a,b,s,z;
	int i;

	if(x < .6) return 1.5707963267948966192 - func_2 (x);
	s = b = 1;
	z = 1 - x;
	for (i = 1; b >= s * 1e-15; i++) {
		b *= z*(2*i-1)/(4*i);
		s += b/(2*i+1);
	}
	return s*sqrt(2*z);
}

static double func_4 (x)
double x; {

	double a,b,c,s;
	int i;

	if(fabs(fabs(x) - 1) < .3) {
		c = x*x;
		a = c/(1+c);
		b = s = 1;
		for (i = 3; fabs(b) >= fabs(s) * 1e-15; ) {
			b *= (i-1)*a/i;
			s += b;
			i += 2;
		}
		s *= x/(1+c);
	} else if(fabs(x) <= 1) {
		s = b = x;
		a = x*x;
		for (i = 3; fabs(b/i) > fabs(s) * 1e-15; ) {
			b *= -a;
			s += b/i;
			i += 2;
		}
	} else {
		b = -1/fabs(x);
		s = func_2(1.) + b;
		a = b*b;
		for (i = 3; fabs(b/i) >= fabs(s) * 1e-15; ) {
			b *= -a;
			s += b/i;
			i += 2;
		}
		if(x < 0) s = - s;
	}
	return s;
}

static void test_arc(void) {
	double x, y, z;
	register unsigned long q = 0;

	for (x = -1; x <= 1; x += 0.03) {
		y = func_2(x);
		z = asin(x);
		if(!y && fabs(z) > 1e-40 ||
		y && fabs((y-z)/y) > 1e-14)
		    printf(t, "ARCSIN", x, y, z), q++;

		y = func_3(x);
		z = acos(x);
		if(!y && fabs(z) > 1e-40 ||
		y && fabs((y-z)/y) > 1e-14)
		    printf(t, "ARCCOS", x, y, z), q++;
	}
	for (x = -10; x <= 10; x += 0.12) {
		y = func_4(x);
		z = atan(x);
		if(!y && fabs(z) > 1e-40 ||
		y && fabs((y-z)/y) > 1e-14)
		    printf(t, "ARCTAN", x, y, z), q++;
	}
	equal(q,0,"ARC tests");
}



/* Test of SINH, COSH und EXP via Taylorreihe */

static char *t1 = "SINH, arg. %f, tay. %f, val. %f\n",
	    *t2 = "COSH, arg. %f, tay. %f, val. %f\n",
	    *t3 = "EXP,  arg. %f, tay. %f, val. %f\n";

double fabs ();

static double h_f1 (x)
double x; {

	register double a,b,s;
	register i;

	if(!x) return 0;
	a = x*x;
	s = b = x;
	for (i = 2; fabs(b) >= fabs(s) * 1e-15; i += 2) {
		b *= a/(i*(i+1.));
		s += b;
	}
	return s;
}

static double h_f2 (x)
double x; {

	register double a,b,s;
	register i;

	if(!x) return 1;
	a = x*x;
	s = b = 1;
	for (i = 1; b >= s * 1e-15; i += 2) {
		b *= a/(i*(i+1.));
		s += b;
	}
	return s;
}

static double h_f3 (x)
double x; {

	register double b,s;
	register i;

	if(!x) return 1;
	s = b = 1;
	for (i = 1; b >= s * 1e-15; ) {
		b *= fabs(x)/i++;
		s += b;
	}
	if(x < 0) s = 1/s;
	return s  ;
}

test_hyper () {
	register double x, y, z;
	double   sinh (), cosh (), exp ();
	register unsigned q = 0;

	for (x = -20; x <= 20; x += 0.062) {
		y = h_f1(x);
		z = sinh(x);
		if(!y && fabs(z) > 1e-40 ||
		y && fabs((y-z)/y) > 1e-14)
		    printf(t1, x, y, z), q++;

		y = h_f2(x);
		z = cosh(x);
		if(fabs((y-z)/y) > 1e-14)
		    printf(t2, x, y, z), q++;

		y = h_f3(x);
		z = exp(x);
		if(!y && fabs(z) > 1e-40 ||
		y && fabs((y-z)/y) > 1e-14)
		    printf(t3, x, y, z), q++;
	}
	equal(q,0,"HYPER");
    }
/*
 * Added 10/15/93; 386 STOS transformation dorqued!!!
 */
int stos_array[100];
test_stos()
    {
    int *p = stos_array;
    int *limit = stos_array+50;
    for (;p<limit;p++) *p = -1;
    equal(stos_array[0],-1,"stos.0");
    equal(stos_array[49],-1,"stos.1");
    equal(stos_array[50],0,"stos.2");
    }

/*
 * Added 10/18/93 by DP
 */
float qfac = 1.0;
struct {
    float cx[3];
    }curr_1 = {{2.0,3.0,4.0}};

float vdot[3] = {5.0,6.0,7};
float fxi2 = 8;
float dtsqi = 9;
struct {
    float ax[3];
    }poten1_1 = {{8,9,10}};
struct {
    float ax1[3];
    }poten2_1 = {{18,19,20}};
float hxsqi = 21, hysqi=22,hxi2=100;
int ij = 2;
int nxd = 0;
static test_more_float(void)
{
float f = qfac * (float)-4. * (curr_1.cx[ij - 2] * 
	(float).25 + curr_1.cx[ij - 1] * (float).5 + 
	curr_1.cx[ij] * (float).25) + hxi2 * (float)4. * (
	vdot[ij] - vdot[ij - 2]) - dtsqi * (poten1_1.ax[ij - 
	1] * (float)8. - poten2_1.ax1[ij - 1] * (float)4.) - 
	hxsqi * (float)2. * (poten1_1.ax[ij] + poten1_1.ax[ij 
	- 2] - poten1_1.ax[ij - 1] * (float)2.) - hysqi * (
	float)2. * (poten1_1.ax[ij + nxd - 1] + poten1_1.ax[
	ij - nxd - 1] - poten1_1.ax[ij - 1] * (float)2.) - 
	hxsqi * (poten2_1.ax1[ij] + poten2_1.ax1[ij - 2] - 
	poten2_1.ax1[ij - 1] * (float)2.) - hysqi * (
	poten2_1.ax1[ij + nxd - 1] + poten2_1.ax1[ij - nxd - 
	1] - poten2_1.ax1[ij - 1] * (float)2.);
dequal(824.0,f,0.0,"moreflt");
}


/*
 * Added 10/19/93 by D.P. Bug caused wave5 benchmark to fail.
 */
void test_ssa2()
{
int j = -100;
int k = 100;
int i;
for (i=0; i < 11; i++){
    int tmp = j;
    j = k;
    k = tmp;
    }
equal(k,-100,"ssa2");
equal(j,100,"ssa2");
}

/*
 * Added 10/20/93 by D.P.
 */
void test_more_float2()
{
static float o3 = 1.0/3.0;
static float one=1.0;
float f = one;
f /= 3.0;
if (f != o3) dequal(f,o3,0.0,"float2");
}

/*
 * Added 10/22/93 by D.P
 */
test_ssa3()
{
int a=1,b=2,c=3;
int i;
for (i=0; i<10; i++){
    int tmp = a;
    a = b;
    b = c;
    c = tmp;
    }
equal(a,2,"ssa3");
equal(b,3,"ssa3");
equal(c,1,"ssa3");
}
/*
 * Added 10/29/93.  Failed on 386 at -O2 and greater.
 */


test_switch_eax() {
	unsigned char red, green, blue;
	hsb2rgb (0.0, 1.0, 1.0, &red, &green, &blue);
	equal(red,255,"switch");
	equal(green,0,"eax");
	equal(blue,0,"eax");
	}

hsb2rgb(H, S, B, r, g, b)
    double      H,
                S,
                B;
    unsigned char    *r,
		     *g,
		     *b;
{
    int         i;
    double      f;
    double      bb;
    unsigned char      p;
    unsigned char      q;
    unsigned char      t;

    H -= floor(H);		/* remove anything over 1 */
    H *= 6.0;
    i = floor(H);		/* 0..5 */
    f = H - (float) i;		/* f = fractional part of H */
    bb = 255.0 * B;
    p = (unsigned char) (bb * (1.0 - S));
    q = (unsigned char) (bb * (1.0 - (S * f)));
    t = (unsigned char) (bb * (1.0 - (S * (1.0 - f))));
    switch (i) {                                                  /* THIS GETS CORRUPTED */
    case 0:
	*r = (unsigned char) bb;
	*g = t;
	*b = p;
	break;
    case 1:
	*r = q;
	*g = (unsigned char) bb;
	*b = p;
	break;
    case 2:
	*r = p;
	*g = (unsigned char) bb;
	*b = t;
	break;
    case 3:
	*r = p;
	*g = q;
	*b = (unsigned char) bb;
	break;
    case 4:
	*r = t;
	*g = p;
	*b = (unsigned char) bb;
	break;
    case 5:
	*r = (unsigned char) bb;
	*g = p;
	*b = q;
	break;
    }
}

/*
 * Added 11/18/93 by D.P.
 * Test received from German company
 */
/* Konversionen vor und nach Multiplikation */

test_german() {
 
	typedef long sl;
	typedef short ss;
	typedef unsigned short us;
	typedef signed char sc;
	typedef unsigned char uc;
 
	static  unsigned long ix [] = { 0xFFFFFFFF, 0x7FFF7F7F,
                                0x80008080, 0x00000001 };
 
	static  long  mx [] = { 0xFFFFFFFF, 16, 0xFFFF, 24, 0xFF }; 
 
	static  char  *t1 = "Konvers. %s, %s und %s, Indizes %d und %d,",
              *t2 [] = {"long","short","unsigned short",
                        "signed char","unsigned char"},
              *t3 = "   %c: %8x, v: %8x\n";
 
 
        unsigned long l, m, n;
        short         q, i, j;
        long          a, b, v;
        short         c, d, k;
 
        long x [5] [5] [4] [5] [4];
        long y [5] [5] [4] [5] [4];
 
        q = 0;
        for (i = 0; i < 4; i++) {
                for (j = 0; j < 4; j++) {
                        a = ix [i];
                        b = ix [j];
                        c = a;
                        d = b;
                        x [0] [0] [i] [0] [j] = (sl) ((sl) a * (sl) b);
                        x [0] [0] [i] [1] [j] = (sl) ((sl) a * (ss) b);
                        x [0] [0] [i] [2] [j] = (sl) ((sl) a * (us) b);
                        x [0] [0] [i] [3] [j] = (sl) ((sl) a * (sc) b);
                        x [0] [0] [i] [4] [j] = (sl) ((sl) a * (uc) b);
                        x [0] [1] [i] [0] [j] = (sl) ((ss) a * (sl) b);
                        x [0] [1] [i] [1] [j] = (sl) ((ss) a * (ss) b);
                        x [0] [1] [i] [2] [j] = (sl) ((ss) a * (us) b);
                        x [0] [1] [i] [3] [j] = (sl) ((ss) a * (sc) b);
                        x [0] [1] [i] [4] [j] = (sl) ((ss) a * (uc) b);
                        x [0] [2] [i] [0] [j] = (sl) ((us) a * (sl) b);
                        x [0] [2] [i] [1] [j] = (sl) ((us) a * (ss) b);
                        x [0] [2] [i] [2] [j] = (sl) ((us) a * (us) b);
                        x [0] [2] [i] [3] [j] = (sl) ((us) a * (sc) b);
                        x [0] [2] [i] [4] [j] = (sl) ((us) a * (uc) b);
                        x [0] [3] [i] [0] [j] = (sl) ((sc) a * (sl) b);
                        x [0] [3] [i] [1] [j] = (sl) ((sc) a * (ss) b);
                        x [0] [3] [i] [2] [j] = (sl) ((sc) a * (us) b);
                        x [0] [3] [i] [3] [j] = (sl) ((sc) a * (sc) b);
                        x [0] [3] [i] [4] [j] = (sl) ((sc) a * (uc) b);
                        x [0] [4] [i] [0] [j] = (sl) ((uc) a * (sl) b);
                        x [0] [4] [i] [1] [j] = (sl) ((uc) a * (ss) b);
                        x [0] [4] [i] [2] [j] = (sl) ((uc) a * (us) b);
                        x [0] [4] [i] [3] [j] = (sl) ((uc) a * (sc) b);
                        x [0] [4] [i] [4] [j] = (sl) ((uc) a * (uc) b);
                        x [1] [0] [i] [0] [j] = (ss) ((sl) a * (sl) b);
                        x [1] [0] [i] [1] [j] = (ss) ((sl) a * (ss) b);
                        x [1] [0] [i] [2] [j] = (ss) ((sl) a * (us) b);
                        x [1] [0] [i] [3] [j] = (ss) ((sl) a * (sc) b);
                        x [1] [0] [i] [4] [j] = (ss) ((sl) a * (uc) b);
                        x [1] [1] [i] [0] [j] = (ss) ((ss) a * (sl) b);
                        x [1] [1] [i] [1] [j] = (ss) ((ss) a * (ss) b);
                        x [1] [1] [i] [2] [j] = (ss) ((ss) a * (us) b);
                        x [1] [1] [i] [3] [j] = (ss) ((ss) a * (sc) b);
                        x [1] [1] [i] [4] [j] = (ss) ((ss) a * (uc) b);
                        x [1] [2] [i] [0] [j] = (ss) ((us) a * (sl) b);
                        x [1] [2] [i] [1] [j] = (ss) ((us) a * (ss) b);
                        x [1] [2] [i] [2] [j] = (ss) ((us) a * (us) b);
                        x [1] [2] [i] [3] [j] = (ss) ((us) a * (sc) b);
                        x [1] [2] [i] [4] [j] = (ss) ((us) a * (uc) b);
                        x [1] [3] [i] [0] [j] = (ss) ((sc) a * (sl) b);
                        x [1] [3] [i] [1] [j] = (ss) ((sc) a * (ss) b);
                        x [1] [3] [i] [2] [j] = (ss) ((sc) a * (us) b);
                        x [1] [3] [i] [3] [j] = (ss) ((sc) a * (sc) b);
                        x [1] [3] [i] [4] [j] = (ss) ((sc) a * (uc) b);
                        x [1] [4] [i] [0] [j] = (ss) ((uc) a * (sl) b);
                        x [1] [4] [i] [1] [j] = (ss) ((uc) a * (ss) b);
                        x [1] [4] [i] [2] [j] = (ss) ((uc) a * (us) b);
                        x [1] [4] [i] [3] [j] = (ss) ((uc) a * (sc) b);
                        x [1] [4] [i] [4] [j] = (ss) ((uc) a * (uc) b);
                        x [2] [0] [i] [0] [j] = (us) ((sl) a * (sl) b);
                        x [2] [0] [i] [1] [j] = (us) ((sl) a * (ss) b);
                        x [2] [0] [i] [2] [j] = (us) ((sl) a * (us) b);
                        x [2] [0] [i] [3] [j] = (us) ((sl) a * (sc) b);
                        x [2] [0] [i] [4] [j] = (us) ((sl) a * (uc) b);
                        x [2] [1] [i] [0] [j] = (us) ((ss) a * (sl) b);
                        x [2] [1] [i] [1] [j] = (us) ((ss) a * (ss) b);
                        x [2] [1] [i] [2] [j] = (us) ((ss) a * (us) b);
                        x [2] [1] [i] [3] [j] = (us) ((ss) a * (sc) b);
                        x [2] [1] [i] [4] [j] = (us) ((ss) a * (uc) b);
                        x [2] [2] [i] [0] [j] = (us) ((us) a * (sl) b);
                        x [2] [2] [i] [1] [j] = (us) ((us) a * (ss) b);
                        x [2] [2] [i] [2] [j] = (us) ((us) a * (us) b);
                        x [2] [2] [i] [3] [j] = (us) ((us) a * (sc) b);
                        x [2] [2] [i] [4] [j] = (us) ((us) a * (uc) b);
                        x [2] [3] [i] [0] [j] = (us) ((sc) a * (sl) b);
                        x [2] [3] [i] [1] [j] = (us) ((sc) a * (ss) b);
                        x [2] [3] [i] [2] [j] = (us) ((sc) a * (us) b);
                        x [2] [3] [i] [3] [j] = (us) ((sc) a * (sc) b);
                        x [2] [3] [i] [4] [j] = (us) ((sc) a * (uc) b);
                        x [2] [4] [i] [0] [j] = (us) ((uc) a * (sl) b);
                        x [2] [4] [i] [1] [j] = (us) ((uc) a * (ss) b);
                        x [2] [4] [i] [2] [j] = (us) ((uc) a * (us) b);
                        x [2] [4] [i] [3] [j] = (us) ((uc) a * (sc) b);
                        x [2] [4] [i] [4] [j] = (us) ((uc) a * (uc) b);
                        x [3] [0] [i] [0] [j] = (sc) ((sl) a * (sl) b);
                        x [3] [0] [i] [1] [j] = (sc) ((sl) a * (ss) b);
                        x [3] [0] [i] [2] [j] = (sc) ((sl) a * (us) b);
                        x [3] [0] [i] [3] [j] = (sc) ((sl) a * (sc) b);
                        x [3] [0] [i] [4] [j] = (sc) ((sl) a * (uc) b);
                        x [3] [1] [i] [0] [j] = (sc) ((ss) a * (sl) b);
                        x [3] [1] [i] [1] [j] = (sc) ((ss) a * (ss) b);
                        x [3] [1] [i] [2] [j] = (sc) ((ss) a * (us) b);
                        x [3] [1] [i] [3] [j] = (sc) ((ss) a * (sc) b);
                        x [3] [1] [i] [4] [j] = (sc) ((ss) a * (uc) b);
                        x [3] [2] [i] [0] [j] = (sc) ((us) a * (sl) b);
                        x [3] [2] [i] [1] [j] = (sc) ((us) a * (ss) b);
                        x [3] [2] [i] [2] [j] = (sc) ((us) a * (us) b);
                        x [3] [2] [i] [3] [j] = (sc) ((us) a * (sc) b);
                        x [3] [2] [i] [4] [j] = (sc) ((us) a * (uc) b);
                        x [3] [3] [i] [0] [j] = (sc) ((sc) a * (sl) b);
                        x [3] [3] [i] [1] [j] = (sc) ((sc) a * (ss) b);
                        x [3] [3] [i] [2] [j] = (sc) ((sc) a * (us) b);
                        x [3] [3] [i] [3] [j] = (sc) ((sc) a * (sc) b);
                        x [3] [3] [i] [4] [j] = (sc) ((sc) a * (uc) b);
                        x [3] [4] [i] [0] [j] = (sc) ((uc) a * (sl) b);
                        x [3] [4] [i] [1] [j] = (sc) ((uc) a * (ss) b);
                        x [3] [4] [i] [2] [j] = (sc) ((uc) a * (us) b);
                        x [3] [4] [i] [3] [j] = (sc) ((uc) a * (sc) b);
                        x [3] [4] [i] [4] [j] = (sc) ((uc) a * (uc) b);
                        x [4] [0] [i] [0] [j] = (uc) ((sl) a * (sl) b);
                        x [4] [0] [i] [1] [j] = (uc) ((sl) a * (ss) b);
                        x [4] [0] [i] [2] [j] = (uc) ((sl) a * (us) b);
                        x [4] [0] [i] [3] [j] = (uc) ((sl) a * (sc) b);
                        x [4] [0] [i] [4] [j] = (uc) ((sl) a * (uc) b);
                        x [4] [1] [i] [0] [j] = (uc) ((ss) a * (sl) b);
                        x [4] [1] [i] [1] [j] = (uc) ((ss) a * (ss) b);
                        x [4] [1] [i] [2] [j] = (uc) ((ss) a * (us) b);
                        x [4] [1] [i] [3] [j] = (uc) ((ss) a * (sc) b);
                        x [4] [1] [i] [4] [j] = (uc) ((ss) a * (uc) b);
                        x [4] [2] [i] [0] [j] = (uc) ((us) a * (sl) b);
                        x [4] [2] [i] [1] [j] = (uc) ((us) a * (ss) b);
                        x [4] [2] [i] [2] [j] = (uc) ((us) a * (us) b);
                        x [4] [2] [i] [3] [j] = (uc) ((us) a * (sc) b);
                        x [4] [2] [i] [4] [j] = (uc) ((us) a * (uc) b);
                        x [4] [3] [i] [0] [j] = (uc) ((sc) a * (sl) b);
                        x [4] [3] [i] [1] [j] = (uc) ((sc) a * (ss) b);
                        x [4] [3] [i] [2] [j] = (uc) ((sc) a * (us) b);
                        x [4] [3] [i] [3] [j] = (uc) ((sc) a * (sc) b);
                        x [4] [3] [i] [4] [j] = (uc) ((sc) a * (uc) b);
                        x [4] [4] [i] [0] [j] = (uc) ((uc) a * (sl) b);
                        x [4] [4] [i] [1] [j] = (uc) ((uc) a * (ss) b);
                        x [4] [4] [i] [2] [j] = (uc) ((uc) a * (us) b);
                        x [4] [4] [i] [3] [j] = (uc) ((uc) a * (sc) b);
                        x [4] [4] [i] [4] [j] = (uc) ((uc) a * (uc) b);
 
                        y [0] [0] [i] [0] [j] = (sl) ((sl) c * (sl) d);
                        y [0] [0] [i] [1] [j] = (sl) ((sl) c * (ss) d);
                        y [0] [0] [i] [2] [j] = (sl) ((sl) c * (us) d);
                        y [0] [0] [i] [3] [j] = (sl) ((sl) c * (sc) d);
                        y [0] [0] [i] [4] [j] = (sl) ((sl) c * (uc) d);
                        y [0] [1] [i] [0] [j] = (sl) ((ss) c * (sl) d);
                        y [0] [1] [i] [1] [j] = (sl) ((ss) c * (ss) d);
                        y [0] [1] [i] [2] [j] = (sl) ((ss) c * (us) d);
                        y [0] [1] [i] [3] [j] = (sl) ((ss) c * (sc) d);
                        y [0] [1] [i] [4] [j] = (sl) ((ss) c * (uc) d);
                        y [0] [2] [i] [0] [j] = (sl) ((us) c * (sl) d);
                        y [0] [2] [i] [1] [j] = (sl) ((us) c * (ss) d);
                        y [0] [2] [i] [2] [j] = (sl) ((us) c * (us) d);
                        y [0] [2] [i] [3] [j] = (sl) ((us) c * (sc) d);
                        y [0] [2] [i] [4] [j] = (sl) ((us) c * (uc) d);
                        y [0] [3] [i] [0] [j] = (sl) ((sc) c * (sl) d);
                        y [0] [3] [i] [1] [j] = (sl) ((sc) c * (ss) d);
                        y [0] [3] [i] [2] [j] = (sl) ((sc) c * (us) d);
                        y [0] [3] [i] [3] [j] = (sl) ((sc) c * (sc) d);
                        y [0] [3] [i] [4] [j] = (sl) ((sc) c * (uc) d);
                        y [0] [4] [i] [0] [j] = (sl) ((uc) c * (sl) d);
                        y [0] [4] [i] [1] [j] = (sl) ((uc) c * (ss) d);
                        y [0] [4] [i] [2] [j] = (sl) ((uc) c * (us) d);
                        y [0] [4] [i] [3] [j] = (sl) ((uc) c * (sc) d);
                        y [0] [4] [i] [4] [j] = (sl) ((uc) c * (uc) d);
                        y [1] [0] [i] [0] [j] = (ss) ((sl) c * (sl) d);
                        y [1] [0] [i] [1] [j] = (ss) ((sl) c * (ss) d);
                        y [1] [0] [i] [2] [j] = (ss) ((sl) c * (us) d);
                        y [1] [0] [i] [3] [j] = (ss) ((sl) c * (sc) d);
                        y [1] [0] [i] [4] [j] = (ss) ((sl) c * (uc) d);
                        y [1] [1] [i] [0] [j] = (ss) ((ss) c * (sl) d);
                        y [1] [1] [i] [1] [j] = (ss) ((ss) c * (ss) d);
                        y [1] [1] [i] [2] [j] = (ss) ((ss) c * (us) d);
                        y [1] [1] [i] [3] [j] = (ss) ((ss) c * (sc) d);
                        y [1] [1] [i] [4] [j] = (ss) ((ss) c * (uc) d);
                        y [1] [2] [i] [0] [j] = (ss) ((us) c * (sl) d);
                        y [1] [2] [i] [1] [j] = (ss) ((us) c * (ss) d);
                        y [1] [2] [i] [2] [j] = (ss) ((us) c * (us) d);
                        y [1] [2] [i] [3] [j] = (ss) ((us) c * (sc) d);
                        y [1] [2] [i] [4] [j] = (ss) ((us) c * (uc) d);
                        y [1] [3] [i] [0] [j] = (ss) ((sc) c * (sl) d);
                        y [1] [3] [i] [1] [j] = (ss) ((sc) c * (ss) d);
                        y [1] [3] [i] [2] [j] = (ss) ((sc) c * (us) d);
                        y [1] [3] [i] [3] [j] = (ss) ((sc) c * (sc) d);
                        y [1] [3] [i] [4] [j] = (ss) ((sc) c * (uc) d);
                        y [1] [4] [i] [0] [j] = (ss) ((uc) c * (sl) d);
                        y [1] [4] [i] [1] [j] = (ss) ((uc) c * (ss) d);
                        y [1] [4] [i] [2] [j] = (ss) ((uc) c * (us) d);
                        y [1] [4] [i] [3] [j] = (ss) ((uc) c * (sc) d);
                        y [1] [4] [i] [4] [j] = (ss) ((uc) c * (uc) d);
                        y [2] [0] [i] [0] [j] = (us) ((sl) c * (sl) d);
                        y [2] [0] [i] [1] [j] = (us) ((sl) c * (ss) d);
                        y [2] [0] [i] [2] [j] = (us) ((sl) c * (us) d);
                        y [2] [0] [i] [3] [j] = (us) ((sl) c * (sc) d);
                        y [2] [0] [i] [4] [j] = (us) ((sl) c * (uc) d);
                        y [2] [1] [i] [0] [j] = (us) ((ss) c * (sl) d);
                        y [2] [1] [i] [1] [j] = (us) ((ss) c * (ss) d);
                        y [2] [1] [i] [2] [j] = (us) ((ss) c * (us) d);
                        y [2] [1] [i] [3] [j] = (us) ((ss) c * (sc) d);
                        y [2] [1] [i] [4] [j] = (us) ((ss) c * (uc) d);
                        y [2] [2] [i] [0] [j] = (us) ((us) c * (sl) d);
                        y [2] [2] [i] [1] [j] = (us) ((us) c * (ss) d);
                        y [2] [2] [i] [2] [j] = (us) ((us) c * (us) d);
                        y [2] [2] [i] [3] [j] = (us) ((us) c * (sc) d);
                        y [2] [2] [i] [4] [j] = (us) ((us) c * (uc) d);
                        y [2] [3] [i] [0] [j] = (us) ((sc) c * (sl) d);
                        y [2] [3] [i] [1] [j] = (us) ((sc) c * (ss) d);
                        y [2] [3] [i] [2] [j] = (us) ((sc) c * (us) d);
                        y [2] [3] [i] [3] [j] = (us) ((sc) c * (sc) d);
                        y [2] [3] [i] [4] [j] = (us) ((sc) c * (uc) d);
                        y [2] [4] [i] [0] [j] = (us) ((uc) c * (sl) d);
                        y [2] [4] [i] [1] [j] = (us) ((uc) c * (ss) d);
                        y [2] [4] [i] [2] [j] = (us) ((uc) c * (us) d);
                        y [2] [4] [i] [3] [j] = (us) ((uc) c * (sc) d);
                        y [2] [4] [i] [4] [j] = (us) ((uc) c * (uc) d);
                        y [3] [0] [i] [0] [j] = (sc) ((sl) c * (sl) d);
                        y [3] [0] [i] [1] [j] = (sc) ((sl) c * (ss) d);
                        y [3] [0] [i] [2] [j] = (sc) ((sl) c * (us) d);
                        y [3] [0] [i] [3] [j] = (sc) ((sl) c * (sc) d);
                        y [3] [0] [i] [4] [j] = (sc) ((sl) c * (uc) d);
                        y [3] [1] [i] [0] [j] = (sc) ((ss) c * (sl) d);
                        y [3] [1] [i] [1] [j] = (sc) ((ss) c * (ss) d);
                        y [3] [1] [i] [2] [j] = (sc) ((ss) c * (us) d);
                        y [3] [1] [i] [3] [j] = (sc) ((ss) c * (sc) d);
                        y [3] [1] [i] [4] [j] = (sc) ((ss) c * (uc) d);
                        y [3] [2] [i] [0] [j] = (sc) ((us) c * (sl) d);
                        y [3] [2] [i] [1] [j] = (sc) ((us) c * (ss) d);
                        y [3] [2] [i] [2] [j] = (sc) ((us) c * (us) d);
                        y [3] [2] [i] [3] [j] = (sc) ((us) c * (sc) d);
                        y [3] [2] [i] [4] [j] = (sc) ((us) c * (uc) d);
                        y [3] [3] [i] [0] [j] = (sc) ((sc) c * (sl) d);
                        y [3] [3] [i] [1] [j] = (sc) ((sc) c * (ss) d);
                        y [3] [3] [i] [2] [j] = (sc) ((sc) c * (us) d);
                        y [3] [3] [i] [3] [j] = (sc) ((sc) c * (sc) d);
                        y [3] [3] [i] [4] [j] = (sc) ((sc) c * (uc) d);
                        y [3] [4] [i] [0] [j] = (sc) ((uc) c * (sl) d);
                        y [3] [4] [i] [1] [j] = (sc) ((uc) c * (ss) d);
                        y [3] [4] [i] [2] [j] = (sc) ((uc) c * (us) d);
                        y [3] [4] [i] [3] [j] = (sc) ((uc) c * (sc) d);
                        y [3] [4] [i] [4] [j] = (sc) ((uc) c * (uc) d);
                        y [4] [0] [i] [0] [j] = (uc) ((sl) c * (sl) d);
                        y [4] [0] [i] [1] [j] = (uc) ((sl) c * (ss) d);
                        y [4] [0] [i] [2] [j] = (uc) ((sl) c * (us) d);
                        y [4] [0] [i] [3] [j] = (uc) ((sl) c * (sc) d);
                        y [4] [0] [i] [4] [j] = (uc) ((sl) c * (uc) d);
                        y [4] [1] [i] [0] [j] = (uc) ((ss) c * (sl) d);
                        y [4] [1] [i] [1] [j] = (uc) ((ss) c * (ss) d);
                        y [4] [1] [i] [2] [j] = (uc) ((ss) c * (us) d);
                        y [4] [1] [i] [3] [j] = (uc) ((ss) c * (sc) d);
                        y [4] [1] [i] [4] [j] = (uc) ((ss) c * (uc) d);
                        y [4] [2] [i] [0] [j] = (uc) ((us) c * (sl) d);
                        y [4] [2] [i] [1] [j] = (uc) ((us) c * (ss) d);
                        y [4] [2] [i] [2] [j] = (uc) ((us) c * (us) d);
                        y [4] [2] [i] [3] [j] = (uc) ((us) c * (sc) d);
                        y [4] [2] [i] [4] [j] = (uc) ((us) c * (uc) d);
                        y [4] [3] [i] [0] [j] = (uc) ((sc) c * (sl) d);
                        y [4] [3] [i] [1] [j] = (uc) ((sc) c * (ss) d);
                        y [4] [3] [i] [2] [j] = (uc) ((sc) c * (us) d);
                        y [4] [3] [i] [3] [j] = (uc) ((sc) c * (sc) d);
                        y [4] [3] [i] [4] [j] = (uc) ((sc) c * (uc) d);
                        y [4] [4] [i] [0] [j] = (uc) ((uc) c * (sl) d);
                        y [4] [4] [i] [1] [j] = (uc) ((uc) c * (ss) d);
                        y [4] [4] [i] [2] [j] = (uc) ((uc) c * (us) d);
                        y [4] [4] [i] [3] [j] = (uc) ((uc) c * (sc) d);
                        y [4] [4] [i] [4] [j] = (uc) ((uc) c * (uc) d);
                }
        }
 
        for (i = 0; i < 4; i++)
            for (j = 0; j < 4; j++)
                for (l = 0; l < 5; l++)
                    for (m = 0; m < 5; m++) {
                        switch (m) {
                        case 0: a = ix [i];
                                break; 
                        case 1:
                        case 3: a = ((long) ix [i] << mx [m]) >> mx [m];
                                break; 
                        case 2:
                        case 4: a = ix [i] & mx [m];
                                break; 
                        }
                        c = a;
 
                        for (n = 0; n < 5; n++) {
                            switch (n) {
                            case 0: b = ix [j];
                                    break; 
                            case 1:
                            case 3: b = ix [j] << mx [n];
                                    b >>= mx [n];
                                    break; 
                            case 2:
                            case 4: b = ix [j] & mx [n];
                                    break; 
                            }
                            d = b;
 
                            for (k = 0; k < 2; k++) {
                                if (!k)
                                    v = a * b;
                                else
                                    v = (m==2?a:c) * (n==2?b:d);
                                switch (l) {
                                case 1:
                                case 3: v <<= mx [l];
                                        v >>= mx [l];
                                        break; 
                                case 2:
                                case 4: v &= mx [l];
                                        break; 
                                }
                                if (   k < 1
                                    && v != x [l] [m] [i] [n] [j]
                                    || k == 1
                                    && v != y [l] [m] [i] [n] [j]) {
                                    if (q++ > 20) {
                                        printf ("zu viele Fehler\n"); 
                                        exit (20); 
                                    }
                                    printf (t1,t2[l],t2[m],t2[n],i,j);
                                    printf (" Quellen %s\n",t2[k]);
                                    printf (t3,k < 1 ? 'x':'y',
                                            k < 1 ? x[l][m][i][n][j]
                                            : y[l][m][i][n][j],v);
                                }
                            }
                        }
                    }
 
	equal(q,0,"german");
    }


/*
 * test_umod()   added: 11/29/93 by D.P.
 */
#pragma on(use_UP_rules);
typedef unsigned long ulong;
 
#define UDIV(R,Z,N)   if ((N) == 1)                              \
                              R = Z;                             \
                      else                                       \
                              if ((N) & 0x80000000)              \
                                      if ((Z) >= (N))            \
                                              R = 1;             \
                                      else                       \
                                              R = 0;             \
                              else                               \
                                      if ((Z) & 0x80000000)      \
                                              R = (ulong) (Z)    \
                                                / (ulong) (N);   \
                                      else                       \
                                              R = (long) (Z)     \
                                                / (long) (N) 
 
#define UMOD(R,Z,N)   if ((N) == 1)                              \
                              R = 0;                             \
                      else                                       \
                              if ((N) & 0x80000000)              \
                                      if ((Z) >= (N))            \
                                              R = (Z) - (N);     \
                                      else                       \
                                              R = Z;             \
                              else                               \
                                      if ((Z) & 0x80000000)      \
                                              R = (ulong) (Z)    \
                                                % (ulong) (N);   \
                                      else                       \
                                              R = (long) (Z)     \
                                                % (long) (N) 
 
test_umod(){

    static unsigned long  a [] = { 0, 5, -5, 8, -8, 15, -15 };
    static unsigned short b [] = { 0, 5, -5, 8, -8, 15, -15 };
    static          short c [] = { 0, 5, -5, 8, -8, 15, -15 };
    static unsigned char  d [] = { 0, 5, -5, 8, -8, 15, -15 };
    static signed   char  e [] = { 0, 5, -5, 8, -8, 15, -15 };
     
    static char *t [ ] = { "( / ) / : r = %d, y2 = %d, %d, %d, %d\n",
                       "( / ) %% : s = %d, y2 = %d, %d, %d, %d\n",
                       "( %% ) / : r = %d, y2 = %d, %d, %d, %d\n",
                       "( %% ) %% : s = %d, y2 = %d, %d, %d, %d\n",
                       " / ( / ) : r = %d, y3 = %d, %d, %d, %d\n",
                       " / ( %% ) : s = %d, y3 = %d, %d, %d, %d\n",
                       " %% ( / ) : r = %d, y3 = %d, %d, %d, %d\n",
                       " %% ( %% ) : s = %d, y3 = %d, %d, %d, %d\n" };
 
 
        short i, j, k, u;
        long  x [14] [7] [7];
        unsigned long y1, y2, y3;
        unsigned long r, s;
        static long w = 15;
 
        for (i = 0; i < 7; i++) {
                x [0] [i] [0] = a [i] % 5;
                x [0] [i] [1] = a [i] % 0xFFFFFFFB;
                x [0] [i] [2] = a [i] % 8;
                x [0] [i] [3] = a [i] % 0xFFFFFFF8;
                x [0] [i] [4] = a [i] % 15;
                x [0] [i] [5] = a [i] % 0xFFFFFFF1;
 
                for (j = 1; j < 7; j++)
                        x [1] [i] [j-1] = a [i] % a [j];
 
                x [2] [i] [0] = (a [i] + 5) % 5;
                x [2] [i] [1] = (a [i] + 5) % 0xFFFFFFFB;
                x [2] [i] [2] = (a [i] + 5) % 8;
                x [2] [i] [3] = (a [i] + 5) % 0xFFFFFFF8;
                x [2] [i] [4] = (a [i] + 5) % 15;
                x [2] [i] [5] = (a [i] + 5) % 0xFFFFFFF1;
 
                for (j = 0; j < 7; j++) {
                        x [3] [i] [j] = a [i] % (a [j] + 10);
                        x [4] [i] [j] = (a [i] + 5) % (a [j] - 7);
                        if (j == 0) continue;
                        x [5] [i] [j] = a [i] % b [j];
                        x [6] [i] [j] = (a [i] + 5) % b [j];
                        x [7] [i] [j] = a [i] % c [j];
                        x [8] [i] [j] = (a [i] + 5) % c [j];
                        x [9] [i] [j] = a [i] % d [j];
                        x [10][i] [j] = (a [i] + 5) % d [j];
                        x [11][i] [j] = a [i] % e [j];
                        x [12][i] [j] = (a [i] + 5) % e [j];
                }
        }
 
        for (i = 0; i < 13 && w > 0; i++)
                for (j = 0; j < 7; j++)
                        for (k = 0; k < 7; k++) {
                                r = a [j];
                                switch (i) {
 
                        case 0:
                        case 1:         if (k == 6) continue;
                                        s = a [k+1];
                                        break;
 
                        case 2:         if (k == 6) continue;
                                        r += 5;
                                        s = a [k+1];
                                        break;
 
                        case 3:         s = a [k] + 10;
                                        break;
 
                        case 4:         r += 5;
                                        s = a [k] - 7;
                                        break;
 
                        default:        if (k == 0) continue;
                                        if (!(i & 1)) r += 5;
                                        switch (i) {
 
                                case 5:
                                case 6:         s = b [k];
                                                break;
 
                                case 7:
                                case 8:         s = c [k];
                                                break;
 
                                case 9:
                                case 10:        s = d [k];
                                                break;
 
                                case 11:
                                default:        s = e [k];
                                                break;
                                        }
                                }
                                UMOD(r,r,s);
                                if (r != x [i] [j] [k]) {
                                        w--;
					equal(r,x[i][j][k],"umod");
                                        printf ("x = %d, ",x[i][j][k]);
                                        printf ("r = %d, %d, %d, %d\n",
                                                r, i, j, k);
                                }
                        }
        u = w;
        if (w < 15)
                printf ("Fehler bei Modulo, einfache Ausdruecke\n");
 
        for (i = 0; i < 7; i++)
                for (j = 1; j < 7; j++) {
                        x [5] [i] [j] = a [i] / b [j];
                        x [6] [i] [j] = (long) a [i] / b [j];
                        x [7] [i] [j] = a [i] / c [j];
                        x [8] [i] [j] = (long) a [i] / c [j];
                        x [9] [i] [j] = a [i] / d [j];
                        x [10][i] [j] = (long) a [i] / d [j];
                        x [11][i] [j] = a [i] / e [j];
                        x [12][i] [j] = (long) a [i] / e [j];
                }
 
        for (i = 5; i < 13 && w > 0; i++)
                for (j = 0; j < 7; j++)
                        for (k = 1; k < 7; k++) {
                                r = a [j];
                                switch (i) {
 
                        case 5:
                        case 6:         s = b [k];
                                        break;
 
                        case 7:
                        case 8:         s = c [k];
                                        break;
 
                        case 9:
                        case 10:        s = d [k];
                                        break;
 
                        case 11:
                        default:        s = e [k];
                                        break;
                                }
                                if (i != 8 && i != 12)
                                        UDIV (r,r,s);
                                else
                                        r = (long) r / (long) s;
                                if (r != x [i] [j] [k]) {
                                        w--;
					equal(r,x[i][j][k],"umod");
                                        printf ("x = %d, ",x[i][j][k]);
                                        printf ("r = %d, %d, %d, %d\n",
                                                r, i, j, k);
                                }
                        }
        if (w < u)
                printf ("Fehler bei Division, einfache Ausdruecke\n");
 
        u = w;
        for (i = 0; i < 7 && w > 0; i++)
                for (j = 1; j < 7 && w > 0; j++)
                        for (k = 1; k < 7; k++) {
                                r = (a [i] / a [j]) / a [k]; 
                                s = (a [i] / a [j]) % a [k]; 
                                UDIV(y1,a [i],a [j]);
                                UDIV(y2,y1,a [k]);
                                if (r != y2){
					equal(r,y2,"umod");
                                        printf (t [0],r,y2,i,j,k),w--;
					}
                                UMOD(y2,y1,a [k]);
                                if (s != y2)
					equal(s,y2,"umod"),
                                        printf (t [1],s,y2,i,j,k),w--;
 
                                r = (a [i] % a [j]) / a [k]; 
                                s = (a [i] % a [j]) % a [k]; 
                                UMOD(y1,a [i],a [j]);
                                UDIV(y2,y1,a [k]);
                                if (r != y2)
					equal(r,y2,"umod"),
                                        printf (t [2],r,y2,i,j,k),w--;
                                UMOD(y2,y1,a [k]);
                                if (s != y2)
					equal(s,y2,"umod"),
                                        printf (t [3],s,y2,i,j,k),w--;
 
                                UDIV(y1,a [j],a [k]);
                                UMOD(y2,a [k],a [k]);
                                if (y1) {
                                        UDIV(y3,a [i],y1);
                                        r = a [i] / (a [j] / a [k]); 
                                        if (r != y3) {
						equal(r,y3,"umod"),
                                                printf(t[4],r,y3,i,j,k);
                                                w--;
                                        }
                                }
                                if (y2) {
                                        UDIV(y3,a [i],y2);
                                        s = a [i] / (a [j] % a [k]); 
                                        if (s != y3) {
						equal(s,y3,"umod"),
                                                printf(t[5],s,y3,i,j,k);
                                                w--;
                                        }
                                }
                                if (y1) {
                                        UMOD(y3,a [i],y1);
                                        r = a [i] % (a [j] / a [k]); 
                                        if (r != y3) {
						equal(r,y3,"umod"),
                                                printf(t[6],r,y3,i,j,k);
                                                w--;
                                        }
                                }
                                if (y2) {
                                        UMOD(y3,a [i],y2);
                                        s = a [i] % (a [j] % a [k]); 
                                        if (s != y3) {
						equal(s,y3,"umod"),
                                                printf(t[7],s,y3,i,j,k);
                                                w--;
                                        }
                                }
                        }
 
        if (w < u)
                printf ("Fehler bei mehrfacher Div. / Mod.\n");
 
        if (w == 15 && !Silent)
                printf ("Test UMOD successful\n");
}
#pragma pop(use_UP_rules);

/*
 * added 12/22/93, raf
 * CC bits getting trashed in 2.5 back-end.
 */
test_CC_optimization() {
    int i,j,k;
    j = k = 0;
    for (i = 0; i < 2; i++) {
	if (j++ % 2) k++;
	}
    equal(k,1,"CC_bits");
    }


/*
 * Added 1/4/94 by DP
 * Fails in all compilers when "behaved" toggle is on (i.e., -O4 or greater).
 */
static void test_behaved()
    {
    struct S{
	int a,b,c,d;
	};

    struct T{
	struct S x,y;
	};

    static struct T t = { {1,2,3,4}, {5,6,7,8} };

    static struct S x = {9,10,11,12};

    static struct T *tp = &t;
    static struct S *xp = &x;

    int ii;
    struct S *p = &tp->y;
    if (tp->y.a == 5){
	*p = *xp;
	}

    equal(tp->y.a,9,"behaved");
    }

/*
 *  Added by D.P. on Jan 6, '94
 *  Shows up flaw in new 387 stuff.
 */
double foo387() {
    static double b=0.0,c = 1.0;
    if (b) {
	equal(0,1,"387FAIL");
	/* Note: nothing returned */
	}
    else{
	double a = b+c+0.5;
	return a;
	}
    }

static void test_387_flaw() {
    dequal(foo387(),1.5,0.0,"387");
    }

/*
 * RAF - Added 1/12/94.
 * old-old opt/csetail.c optimization did not work
 * correctly in all cases. Assignment to vptr0 was
 * being treated as a CSE when in fact it is not.
 */
double chaaxis(int npts,double *vptr) {
    double incval;
    int i;
    double *vptr0;
    incval = -1.0;
    for (i = 0, vptr0 = vptr; (i < npts); i++, vptr++ ) {
        if (i != 0) {
            incval = (((double) (*vptr-*vptr0)) < ((double) (incval)) ?
              ((double) (incval)) :
              ((double) (*vptr - *vptr0)));
            }
        vptr0 = vptr;
        }
    return incval;
    }

static void test_csetail_bug() {
    double vek[] = {1.0,2.0,3.0};
    dequal(chaaxis(sizeof(vek)/sizeof(vek[0]),vek),1.0,0.0,"csetail");
    }

/*
 * ADDED BY D.P. ON 1/25/94. Failed on 386 at -O
 */
void test_loop ()
{
  unsigned int i;
  static int a[] = {0,1,2,3,4,5,6,7};
  int j;
  int sum;

  sum = 0;

  j = 5;
  for (i = 4294967295 - 1; i > 5; i++)
    {
      sum += a[j];
      j++;
    }
  equal(sum,11,"loop");
}

 
/*
 * ADDED BY BUDI ON 1/25/94. Failed on SPARC at -O2
 */
char *argstring(void);
 
unsigned long rmany=1;
unsigned long rmax=0xffff;
unsigned long rmin=0;
unsigned long pse=0;

test_6955()
{
  char *c;
  unsigned long i,r,rng,col;
  double d;
 
  if ((c=argstring()) != 0) 
  {
    d=atof(c)*1000000.0;
    pse = d;
  }
  else pse=0;
 
  rng=abs( (int)(rmax-rmin));
 
  col=0;
 
  for ( i=0; i<rmany; i++ )
  {
    r=(1234*rng)/0xffff;
    printf("",r); /*Does nothing; but materializes the bug*/
  };

  equal(i,1,"6955_i");
  equal(rmany,1,"6955_r");
 }
 
char *argstring(void)
{
    return( (char *) 0 );
}

/*
 * added 1/27/94, raf
 * Sunpro has reported a bug to us where generated C code has 708 arguments
 * to a function call. We used to have an implicit limit of 255 arguments
 * which was causing some interesting behavior when there were more than
 * 255 (Sunpro was getting a premature EOL in the intermediate file). To test
 * the fix out, this code uses 621 arguments. Sorry for the size, but there
 * is not much else we can do.
 */
void check_args(int i000,int i001,int i002,int i003,int i004,int i005,int i006,
                int i007,int i008,int i009,int i010,int i011,int i012,int i013,
                int i014,int i015,int i016,int i017,int i018,int i019,int i020,
                int i021,int i022,int i023,int i024,int i025,int i026,int i027,
                int i028,int i029,int i030,int i031,int i032,int i033,int i034,
                int i035,int i036,int i037,int i038,int i039,int i040,int i041,
                int i042,int i043,int i044,int i045,int i046,int i047,int i048,
                int i049,int i050,int i051,int i052,int i053,int i054,int i055,
                int i056,int i057,int i058,int i059,int i060,int i061,int i062,
                int i063,int i064,int i065,int i066,int i067,int i068,int i069,
                int i070,int i071,int i072,int i073,int i074,int i075,int i076,
                int i077,int i078,int i079,int i080,int i081,int i082,int i083,
                int i084,int i085,int i086,int i087,int i088,int i089,int i090,
                int i091,int i092,int i093,int i094,int i095,int i096,int i097,
                int i098,int i099,int i100,int i101,int i102,int i103,int i104,
                int i105,int i106,int i107,int i108,int i109,int i110,int i111,
                int i112,int i113,int i114,int i115,int i116,int i117,int i118,
                int i119,int i120,int i121,int i122,int i123,int i124,int i125,
                int i126,int i127,int i128,int i129,int i130,int i131,int i132,
                int i133,int i134,int i135,int i136,int i137,int i138,int i139,
                int i140,int i141,int i142,int i143,int i144,int i145,int i146,
                int i147,int i148,int i149,int i150,int i151,int i152,int i153,
                int i154,int i155,int i156,int i157,int i158,int i159,int i160,
                int i161,int i162,int i163,int i164,int i165,int i166,int i167,
                int i168,int i169,int i170,int i171,int i172,int i173,int i174,
                int i175,int i176,int i177,int i178,int i179,int i180,int i181,
                int i182,int i183,int i184,int i185,int i186,int i187,int i188,
                int i189,int i190,int i191,int i192,int i193,int i194,int i195,
                int i196,int i197,int i198,int i199,int i200,int i201,int i202,
                int i203,int i204,int i205,int i206,int i207,int i208,int i209,
                int i210,int i211,int i212,int i213,int i214,int i215,int i216,
                int i217,int i218,int i219,int i220,int i221,int i222,int i223,
                int i224,int i225,int i226,int i227,int i228,int i229,int i230,
                int i231,int i232,int i233,int i234,int i235,int i236,int i237,
                int i238,int i239,int i240,int i241,int i242,int i243,int i244,
                int i245,int i246,int i247,int i248,int i249,int i250,int i251,
                int i252,int i253,int i254,int i255,int i256,int i257,int i258,
                int i259,int i260,int i261,int i262,int i263,int i264,int i265,
                int i266,int i267,int i268,int i269,int i270,int i271,int i272,
                int i273,int i274,int i275,int i276,int i277,int i278,int i279,
                int i280,int i281,int i282,int i283,int i284,int i285,int i286,
                int i287,int i288,int i289,int i290,int i291,int i292,int i293,
                int i294,int i295,int i296,int i297,int i298,int i299,int i300,
                int i301,int i302,int i303,int i304,int i305,int i306,int i307,
                int i308,int i309,int i310,int i311,int i312,int i313,int i314,
                int i315,int i316,int i317,int i318,int i319,int i320,int i321,
                int i322,int i323,int i324,int i325,int i326,int i327,int i328,
                int i329,int i330,int i331,int i332,int i333,int i334,int i335,
                int i336,int i337,int i338,int i339,int i340,int i341,int i342,
                int i343,int i344,int i345,int i346,int i347,int i348,int i349,
                int i350,int i351,int i352,int i353,int i354,int i355,int i356,
                int i357,int i358,int i359,int i360,int i361,int i362,int i363,
                int i364,int i365,int i366,int i367,int i368,int i369,int i370,
                int i371,int i372,int i373,int i374,int i375,int i376,int i377,
                int i378,int i379,int i380,int i381,int i382,int i383,int i384,
                int i385,int j386,int i387,int i388,int i389,int i390,int i391,
                int i392,int i393,int i394,int i395,int i396,int i397,int i398,
                int i399,int i400,int i401,int i402,int i403,int i404,int i405,
                int i406,int i407,int i408,int i409,int i410,int i411,int i412,
                int i413,int i414,int i415,int i416,int i417,int i418,int i419,
                int i420,int i421,int i422,int i423,int i424,int i425,int i426,
                int i427,int i428,int i429,int i430,int i431,int i432,int i433,
                int i434,int i435,int i436,int i437,int i438,int i439,int i440,
                int i441,int i442,int i443,int i444,int i445,int i446,int i447,
                int i448,int i449,int i450,int i451,int i452,int i453,int i454,
                int i455,int i456,int i457,int i458,int i459,int i460,int i461,
                int i462,int i463,int i464,int i465,int i466,int i467,int i468,
                int i469,int i470,int i471,int i472,int i473,int i474,int i475,
                int i476,int i477,int i478,int i479,int i480,int i481,int i482,
                int i483,int i484,int i485,int i486,int i487,int i488,int i489,
                int i490,int i491,int i492,int i493,int i494,int i495,int i496,
                int i497,int i498,int i499,int i500,int i501,int i502,int i503,
                int i504,int i505,int i506,int i507,int i508,int i509,int i510,
                int i511,int i512,int i513,int i514,int i515,int i516,int i517,
                int i518,int i519,int i520,int i521,int i522,int i523,int i524,
                int i525,int i526,int i527,int i528,int i529,int i530,int i531,
                int i532,int i533,int i534,int i535,int i536,int i537,int i538,
                int i539,int i540,int i541,int i542,int i543,int i544,int i545,
                int i546,int i547,int i548,int i549,int i550,int i551,int i552,
                int i553,int i554,int i555,int i556,int i557,int i558,int i559,
                int i560,int i561,int i562,int i563,int i564,int i565,int i566,
                int i567,int i568,int i569,int i570,int i571,int i572,int i573,
                int i574,int i575,int i576,int i577,int i578,int i579,int i580,
                int i581,int i582,int i583,int i584,int i585,int i586,int i587,
                int i588,int i589,int i590,int i591,int i592,int i593,int i594,
                int i595,int i596,int i597,int i598,int i599,int i600,int i601,
                int i602,int i603,int i604,int i605,int i606,int i607,int i608,
                int i609,int i610,int i611,int i612,int i613,int i614,int i615,
                int i616,int i617,int i618,int i619,int i620) {
    int no_inlining[16384];
    equal(i375,375,"i375"); equal(i441,441,"i441"); equal(i597,597,"i597"); 
    equal(i618,618,"i618"); equal(i619,619,"i619"); equal(i620,620,"i620");
    return;
    }

void test_many_args() {
    check_args(  0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15,
                16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31,
                32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47,
                48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63,
                64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79,
                80, 81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95,
                96, 97, 98, 99,100,101,102,103,104,105,106,107,108,109,110,111,
               112,113,114,115,116,117,118,119,120,121,122,123,124,125,126,127,
               128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143,
               144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159,
               160,161,162,163,164,165,166,167,168,169,170,171,172,173,174,175,
               176,177,178,179,180,181,182,183,184,185,186,187,188,189,190,191,
               192,193,194,195,196,197,198,199,200,201,202,203,204,205,206,207,
               208,209,210,211,212,213,214,215,216,217,218,219,220,221,222,223,
               224,225,226,227,228,229,230,231,232,233,234,235,236,237,238,239,
               240,241,242,243,244,245,246,247,248,249,250,251,252,253,254,255,
               256,257,258,259,260,261,262,263,264,265,266,267,268,269,270,271,
               272,273,274,275,276,277,278,279,280,281,282,283,284,285,286,287,
               288,289,290,291,292,293,294,295,296,297,298,299,300,301,302,303,
               304,305,306,307,308,309,310,311,312,313,314,315,316,317,318,319,
               320,321,322,323,324,325,326,327,328,329,330,331,332,333,334,335,
               336,337,338,339,340,341,342,343,344,345,346,347,348,349,350,351,
               352,353,354,355,356,357,358,359,360,361,362,363,364,365,366,367,
               368,369,370,371,372,373,374,375,376,377,378,379,380,381,382,383,
               384,385,386,387,388,389,390,391,392,393,394,395,396,397,398,399,
               400,401,402,403,404,405,406,407,408,409,410,411,412,413,414,415,
               416,417,418,419,420,421,422,423,424,425,426,427,428,429,430,431,
               432,433,434,435,436,437,438,439,440,441,442,443,444,445,446,447,
               448,449,450,451,452,453,454,455,456,457,458,459,460,461,462,463,
               464,465,466,467,468,469,470,471,472,473,474,475,476,477,478,479,
               480,481,482,483,484,485,486,487,488,489,490,491,492,493,494,495,
               496,497,498,499,500,501,502,503,504,505,506,507,508,509,510,511,
               512,513,514,515,516,517,518,519,520,521,522,523,524,525,526,527,
               528,529,530,531,532,533,534,535,536,537,538,539,540,541,542,543,
               544,545,546,547,548,549,550,551,552,553,554,555,556,557,558,559,
               560,561,562,563,564,565,566,567,568,569,570,571,572,573,574,575,
               576,577,578,579,580,581,582,583,584,585,586,587,588,589,590,591,
               592,593,594,595,596,597,598,599,600,601,602,603,604,605,606,607,
               608,609,610,611,612,613,614,615,616,617,618,619,620);
    return;
    }

/*
 * Added Feb 18, 1994 by D.P.
 * Caused HLIL-to-LLIL translation to emit infinite size branch table.
 */
static void test_switch()
    {
    int i = INT_MAX;
    switch(i){
	case INT_MIN:
	case INT_MIN+1:
	case INT_MIN+2:
	case INT_MIN+3:
	case INT_MIN+4:
	case INT_MAX-1:
	case INT_MAX-2:
	case INT_MAX-3:
	case INT_MAX-4:
	    equal(1,0,"switch");
	    break;
	case INT_MAX:
	    equal(i,INT_MAX,"switch");
	    break;
	default:
	    equal(2,1,"switch");
	    break;
	}
    }

main(int argc) {
    announce("START OF TESTS, suite8");
    Silent = argc <= 1;
    #if ANNOUNCE_TESTS
    #define Z(x) (printf("%s\n",#x), x)
    #else
    #define Z(x) x
    #endif
    Z(test_387());
    Z(test_if());
    Z(test_cc());
    Z(test_inline());
    Z(test_floatint());
    Z(test_fltfold());
    Z(test_byte());
    Z(test_longstring());
    Z(test_fwdinline());
    Z(test_387_again());
    Z(test_387_yet_again());
    Z(test_amd_structs());
    Z(test_amd_structs2());
    Z(test_bits());
    Z(test_wierd_loops());
    Z(test_valnum());
    Z(test_fltmat());
    Z(test_induct());
    Z(test_float_prec());
    Z(test_atan());
    Z(test_inline1());
    Z(test_ssa1());
    Z(test_arc());
    Z(test_hyper());
    Z(test_stos());
    Z(test_more_float());
    Z(test_ssa2());
    Z(test_more_float2());
    Z(test_ssa3());
    Z(test_switch_eax());
    Z(test_german());
    Z(test_umod());
    Z(test_CC_optimization());
    Z(test_behaved());
    Z(test_387_flaw());
    Z(test_csetail_bug());
    Z(test_loop());
    Z(test_6955());
    Z(test_many_args());
    Z(test_switch());
    announce("END OF TESTS, suite8");
    exit(0);
    }
