/*
 * The last line is an instance.
 */

 




typedef unsigned long ulong;
typedef unsigned int uint;
typedef unsigned short ushort;
typedef unsigned char uchar;
typedef unsigned char bool;
typedef void (*PFV)();
typedef int (*PFI)();
typedef long (*PFL)();
 
 

enum {
 SYS_EVENT,
 SCRIPT_EVENT
};

 
enum {
 WINDOW_RESIZED = (0x80000000L),
 WINDOW_MOVED,
 CLOSE_REQUESTED,
 ACTIVATION,
 MOUSE_DOWN,
 MOUSE_UP,
 MOUSE_MOVED,
 PARCEL_DROPPED,
 TICK,
 KEY_DOWN,
 VALUE_CHANGED,
 VIEW_RESIZED,
 VIEW_MOVED,
 KEY_UP,
 PUBLIC_EVENT_END = (WINDOW_RESIZED + 0xffff)
};

 

enum {
 UNKNOWN_AREA,
 TITLE_BAR,
 CONTENT_AREA,
 RESIZE_AREA,
 CLOSE_BOX
};

 

enum {
 NO_MEMORY = (0x80000000L),  
 IO_ERROR,   
 PERMISSION_DENIED,  

 FILE_ERROR,   
 FILE_NOT_FOUND,   

 BAD_ARG_TYPE,   
 BAD_ARG_VALUE,   

 NAME_NOT_FOUND,   
 NAME_IN_USE,   

 
 BAD_SEM_ID,   
 NO_MORE_SEMS,   

 
 BAD_THREAD_ID,   
   NO_MORE_THREADS,  
 WRONG_THREAD_STATE,    

 
 BAD_PORT_ID,   
 NO_MORE_PORTS,   
 PORT_BUSY,   

 
 BAD_RESOURCE_ID,  
 NO_ARCHIVE,   
 DUPLICATE_RESOURCE,  

 
 MODEM_ERROR,
 MODEM_BUSY,
 MODEM_NO_CARRIER,

 
 DATA_PORT_ERROR,

 
 BE_ERROR_END = (NO_MEMORY + 0xffff),

 SYS_ERROR = -1,   
 NO_ERROR = 0
};

 
enum {
 CHAR_TYPE,
 SHORT_TYPE,
 LONG_TYPE,
 UCHAR_TYPE,
 USHORT_TYPE,
 ULONG_TYPE, 
 FLOAT_TYPE,
 DOUBLE_TYPE,
 POINT_TYPE,
 RECT_TYPE,
 RGB_COLOR_TYPE,
 PATTERN_TYPE,
 ASCII_TYPE,
 RTF_TYPE,
 STRING_TYPE,
 MONOCHROME_1_BIT_TYPE,
 GRAYSCALE_8_BIT_TYPE,
 COLOR_8_BIT_TYPE,
 RGB_24_BIT_TYPE,
 TIFF_TYPE,
 FILE_TYPE,
 PARCEL_TYPE,
 ALL_TYPES
};
class BPoint {

public:
 long x;
 long y;

 
 BPoint();
 BPoint(long X, long Y);
 BPoint(const BPoint& pt);
 
 
 void  Set(long X, long Y);

 
 void  PrintToStream();
 
 
 BPoint  operator+(const BPoint&) const;
 BPoint  operator-(const BPoint&) const;
 BPoint&  operator+=(const BPoint&);
 BPoint&  operator-=(const BPoint&);

 
 bool  operator!=(const BPoint&) const;
 bool  operator==(const BPoint&) const;
 
 
 void  ScriptGetPackLen( long* runningLen );
 void  ScriptPack( void** runningData );
 void  ScriptUnpack( void** runningData );
};


inline BPoint::BPoint()
{
}

inline BPoint::BPoint(long X, long Y)
{
 x = X;
 y = Y;
}

inline BPoint::BPoint(const BPoint& pt)
{
 x = pt.x;
 y = pt.y;
}

inline void BPoint::Set(long X, long Y)
{
 x = X;
 y = Y;
}

void   ScriptGetLongPackLen( long* runningLength );
void   ScriptPackLong( void** runningData, long data );
long   ScriptUnpackLong( void** runningData );

inline void BPoint::ScriptGetPackLen( long* runningLen ){
 ScriptGetLongPackLen( runningLen );
 ScriptGetLongPackLen( runningLen );
}
inline void BPoint::ScriptPack( void** runningData ){
 ScriptPackLong( runningData, x );
 ScriptPackLong( runningData, y );
}

inline void BPoint::ScriptUnpack( void** runningData ){
 x = ScriptUnpackLong( runningData );
 y = ScriptUnpackLong( runningData );
}
class BRect {

public:
 long  left;
 long  top;
 long  right;
 long  bottom;

 
 BRect();
 BRect(const BRect *);
 BRect(long l, long t, long r, long b);
 BRect(BPoint leftTop, BPoint rightBottom);

 
 void  Set(long l, long t, long r, long b);

 
 void  PrintToStream() const;

 
 BPoint  LeftTop() const;
 BPoint  RightBottom() const;
 BPoint  LeftBottom() const;
 BPoint  RightTop() const;

 
 void  SetLeftTop(const BPoint);
 void  SetRightBottom(const BPoint);
 void  SetLeftBottom(const BPoint);
 void  SetRightTop(const BPoint);

 
 void  InsetBy(long dx, long dy);
 void  OffsetBy(long dx, long dy);
 void  OffsetTo(BPoint);
 void  OffsetTo(long x, long y);

 
 bool  operator==(const BRect&) const;
 bool  operator!=(const BRect&) const;

 
 BRect  operator&(const BRect&) const;
 BRect  operator|(const BRect&) const;

 
 bool  Intersects(const BRect* r) const;
 bool  IsValid() const;
 long  Width() const;
 long  Height() const;
 bool  Contains(BPoint) const;
 bool  Contains(const BRect *) const;
 void  ConstrainPoint(BPoint *);
 
 
 void  ScriptGetPackLen( long* runningLen );
 void  ScriptPack( void** runningData );
 void  ScriptUnpack( void** runningData );
};




inline BPoint BRect::LeftTop() const
{
 return(*((BPoint*)&left));
}

inline BPoint BRect::RightBottom() const
{
 return(*((BPoint*)&right));
}

inline BPoint BRect::LeftBottom() const
{
 return(BPoint(left, bottom));
}

inline BPoint BRect::RightTop() const
{
 return(BPoint(right, top));
}

inline BRect::BRect()
{
 
 top = left = 0;
 bottom = right = -1;
}

inline BRect::BRect(long l, long t, long r, long b)
{
 left = l;
 top = t;
 right = r;
 bottom = b;
}

inline BRect::BRect(const BRect* r)
{
 left = r->left;
 top = r->top;
 right = r->right;
 bottom = r->bottom;
}

inline BRect::BRect(BPoint leftTop, BPoint rightBottom)
{
 left = leftTop.x;
 top = leftTop.y;
 right = rightBottom.x;
 bottom = rightBottom.y;
}

inline void BRect::Set(long l, long t, long r, long b)
{
 left = l;
 top = t;
 right = r;
 bottom = b;
}

inline bool BRect::IsValid() const
{
 if (left <= right && top <= bottom)
  return 1   ;
 else
  return 0    ;
}

inline long BRect::Width() const
{
 return(right - left + 1);
}

inline long BRect::Height() const
{
 return(bottom - top + 1);
}

void   ScriptGetLongPackLen( long* runningLength );
void   ScriptPackLong( void** runningData, long data );
long   ScriptUnpackLong( void** runningData );

inline void BRect::ScriptGetPackLen( long* runningLen ){
 ScriptGetLongPackLen( runningLen );
 ScriptGetLongPackLen( runningLen );
 ScriptGetLongPackLen( runningLen );
 ScriptGetLongPackLen( runningLen );
}
inline void BRect::ScriptPack( void** runningData ){
 ScriptPackLong( runningData, left );
 ScriptPackLong( runningData, top );
 ScriptPackLong( runningData, right );
 ScriptPackLong( runningData, bottom );
}

inline void BRect::ScriptUnpack( void** runningData ){
 left = ScriptUnpackLong( runningData );
 top = ScriptUnpackLong( runningData );
 right = ScriptUnpackLong( runningData );
 bottom = ScriptUnpackLong( runningData );
}
BRect recto;
