//******************************************************************************
//
//	File:		main.cpp
//
//	Description:	text editor program.
//
//	Written by:	Eric Knight
//
//	Copyright 1993, Be Incorporated
//
//	Change History:
//
//	9/6/92		ehk	new today
//	11/08/93	jdr	started hacking
//
//******************************************************************************


#ifndef __EDITOR__
#include "main.h"
#endif 

#ifndef __MODAL__
#include "modal.h"
#endif

#ifndef INTERFACE_KIT_H
#include "InterfaceKit.h"
#endif

#ifndef OPEN_H
#include "open.h"
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

//-----------------------------------------------------------------------------
// GLOBALS
//-----------------------------------------------------------------------------

BApplication*	my_app;		// App must be global
TDocWindow*	my_window;	// Globals are sometimes necessary

static void SetMenuItemMark(TMenuBar* mbar, long itemCode, char state)
{
	if (state)
		mbar->set_mark(itemCode, '*');
	else
		mbar->set_mark(itemCode, ' ');

}

//-------------------------------------------------------------------
// M Y S F D I A L O G
//-------------------------------------------------------------------
void	MySFDialog::MySFDialog( char *title, bool save ) :
	FileOpenWindow( title, save )
{
};

//-------------------------------------------------------------------
void	MySFDialog::FileOpened(long volID, long dirID, char *fileName)
{
	setcurvoldir(volID, dirID);
	my_window->fVolID = volID;
	my_window->fDirID = dirID;
	my_window->fEditView->SelectAll();
	my_window->fEditView->PasteFile(fileName);
	my_window->SetTitle(fileName);
	my_window->checksum = my_window->ComputeChecksum();
	my_window->fEditView->Select(0,0);
	this->Close();
}

//-------------------------------------------------------------------
void	MySFDialog::FileSaved(long volID, long dirID, char *fileName)
{
	my_window->fVolID = volID;
	my_window->fDirID = dirID;
	my_window->SaveFile( fileName );
	this->Close();
}

//-------------------------------------------------------------------
// M Y T E X T V I E W
//-------------------------------------------------------------------

void	MyTextView::MyTextView(	BRect *frame, char *name, BRect *textRect,
				long resizeMode, long flags) :
	BTextView( frame, name, textRect, resizeMode, flags)
{
	fSearchString = malloc(64);
	*fSearchString = 0;
	fCaseSensitive = FALSE;
};

//----------------------------------------------------------------------
// SearchFor-- This routine actually implements the search. Ugly brute
//	       force search.
//----------------------------------------------------------------------
bool	MyTextView::SearchFor( char *target, long startPosition, short direction )
{
	char	*theText;
	long	matchPos, startPos;

	long	i,j;
	long	m = strlen(target);
	long	n = ( this->TextLength() - m );
	
	if ( m > 0 )			// Don't search for nil text
	{
	
		theText = this->Text();		// Get (null terminated) ptr to text in view

		//
		// Determine if we have an existing selection (selStart == selEnd)
		// If we do, go on space past the start; otherwise use the start
		//
	
		if ( SelectionStart() == SelectionEnd())
			startPos = SelectionStart();
		else
			startPos = SelectionStart() + 1;
			
		//
		// Check for trivial abort if current insertion point is
		// less than strlen( target ) characters from the end of the
		// file
		//
		if ( (n - m) < startPos )
			return FALSE;
			
		for (i = startPos, j = 0; j < m && i <= n; i++, j++)
			while (( theText[i] != target[j]) && ( i <= n ))
			{
				i -= j-1;
				j = 0;
			}
		if ( j == m ) 
			matchPos = i - m;
		else
			matchPos = i;

		if ( matchPos >= n )
		{
			TellUser("Not found!");
			return FALSE;
		}

		if ( matchPos < ( n - strlen(target) ))
		{
			this->Select( matchPos, matchPos + strlen( target ));
			this->ScrollToSelection();
			return TRUE;
		}
		else
			return FALSE;
	}
	else
		return FALSE;
};
	
//-----------------------------------------------------------------------
bool	MyTextView::ParcelDropped( BParcel *parcel )
{
	char		*data;
	char		fname[32];
	long		size;
	long		type;
	long		result;
	long		vol;
	long		dir;
	
	result = parcel->GetItem((long)'FBMP', 0, &(void *)data, &size, &type);
	if (result) 
	{
		if ( my_window->OKToOpenOrQuit() )
		{
			memcpy(fname, (char *)&(data[8]), 20);
			memcpy(&vol, (char *)&(data[0]), 4);
			memcpy(&dir, (char *)&(data[4]), 4);
			//getcurvoldir(&old_vol, &old_dir);
			setcurvoldir(vol, dir);
			my_window->fVolID = vol;
			my_window->fDirID = dir;
			this->SelectAll();
			this->PasteFile(fname);
			my_window->SetTitle(fname);
			my_window->checksum = my_window->ComputeChecksum();
			my_window->fEditView->Select(0,0);
			//setcurvoldir(old_vol, old_dir);
		}
	}

	return(TRUE);
}

//----------------------------------------------------------------------
// SetCapsSensitive-- Turn case sensitivity in searches on or off
//----------------------------------------------------------------------
void	MyTextView::CaseSensitive( bool option )
{
	this->fCaseSensitive = option;
}

//-------------------------------------------------------------------
// T D O C W I N D O W
//-------------------------------------------------------------------

void	TDocWindow::TDocWindow(char* fileName, BRect *bound, window_type type, long flags, char *name) :
	BWindow(bound, name, type, flags )
{
	TMenu*		a_menu;
	BRect		a_rect;
	
	getcurvoldir(&fVolID, &fDirID);
	this->checksum = 0l;
	a_rect.Set( 0, 0, 1000, 13 );
	AddChild(fMenuBar = new TMenuBar("MB", &a_rect, FOLLOW_LEFT_TOP | WILL_DRAW));
	
	a_menu = new TMenu("Be", fMenuBar);
	a_menu->add_item("About Edit...", ABOUT);

	a_menu = new TMenu("File", fMenuBar);
	a_menu->add_item("New", NEW);
	a_menu->add_item("Open...", OPEN);
	a_menu->add_item("Save", SAVE);
	a_menu->add_item("Save As...", SAVEAS);
	a_menu->add_item("-", 0);
	a_menu->add_item("Quit", QUIT);
	
	a_menu = new TMenu("Edit", fMenuBar);
	a_menu->add_item("Cut", CUT);
	a_menu->add_item("Copy", COPY);
	a_menu->add_item("Paste", PASTE);
	a_menu->add_item("Delete", DELETE);
	a_menu->add_item("-", 0);
	a_menu->add_item("Select All", SELECTALL);
	a_menu->add_item("-",0);
	a_menu->add_item("Find...",SEARCH);
	a_menu->add_item("Find Again",FINDAGAIN);
	a_menu->add_item("Replace...",REPLACE);
	a_menu->add_item("-",0);
	a_menu->add_item("Jump to top",JUMPTOP);
	a_menu->add_item("Jump to bottom",JUMPBOTTOM);
	
	// Set up the scrollview and textview
	a_rect.Set( 0, 14, kWindowH - 12, kWindowV - 12 );
	fEditView = new MyTextView(&a_rect, "TE", &a_rect, FOLLOW_ALL_SIDES, WILL_DRAW);

	BScrollView	*theScrollView;
	theScrollView = new BScrollView("TS", fEditView, FOLLOW_ALL_SIDES, 0,
				 TRUE, TRUE, FALSE);
	AddChild( theScrollView );	
	
	fEditView->MakeFocus();	
	fEditView->SetFontName("m");  // Monaco
	this->fTextSize = TWELVE;
	fEditView->SetFontSize( 12 );
	fEditView->SetTabWidth( 55 );	// Be standard (approx. 8 chars )
	
	a_menu = new TMenu("Font", fMenuBar);
	a_menu->add_item("Under construction...", 99);
	
	/*	Not in this release
	a_menu = new TMenu("Size", fMenuBar);
	a_menu->add_item("9 points", NINE);
	a_menu->add_item("10 points", TEN);
	a_menu->add_item("12 points", TWELVE);
	a_menu->add_item("14 points", FOURTEEN);
	a_menu->add_item("18 points", EIGHTEEN);
	a_menu->add_item("20 points", TWENTY);
	a_menu->add_item("24 points", TWENTYFOUR);
	
	SetMenuItemMark(fMenuBar, TWELVE, TRUE);
	*/

	a_menu = new TMenu("Settings", fMenuBar);
	a_menu->add_item("Left Justify", LEFT);
	a_menu->add_item("Right Justify", RIGHT);
	a_menu->add_item("Center Justify", CENTER);
	SetMenuItemMark(fMenuBar, LEFT + fEditView->Alignment(), TRUE);
	
	a_menu->add_item("-", 0);
	a_menu->add_item("Single Spacing", SINGLE);
	a_menu->add_item("Double Spacing", DOUBLE);
	SetMenuItemMark(fMenuBar, SINGLE + fEditView->Spacing() - 1, TRUE);
	
	a_menu->add_item("-", 0);
	a_menu->add_item("Wrap Words", WRAP);
	a_menu->add_item("-", 0);
	a_menu->add_item("Auto Indent", AUTO);
	SetMenuItemMark(fMenuBar, WRAP, fEditView->DoesWordWrap());
	SetMenuItemMark(fMenuBar, AUTO, fEditView->DoesAutoindent());

	//
	// Now add command keys for the various items
	//
	this->AddKeyCommand( 'f', ALT_KEY, SEARCH );
	this->AddKeyCommand( 'g', ALT_KEY, FINDAGAIN );
	this->AddKeyCommand( 'q', ALT_KEY, QUIT );
	this->AddKeyCommand( 'o', ALT_KEY, OPEN );
	this->AddKeyCommand( 's', ALT_KEY, SAVE );
	this->AddKeyCommand( 't', ALT_KEY, JUMPTOP );
	this->AddKeyCommand( 'b', ALT_KEY, JUMPBOTTOM );
	this->AddKeyCommand( 'a', ALT_KEY, SELECTALL );
	this->AddKeyCommand( 'x', ALT_KEY, CUT );
	this->AddKeyCommand( 'c', ALT_KEY, COPY );
	this->AddKeyCommand( 'v', ALT_KEY, PASTE );
	
	if (fileName == NIL) {
		SetTitle("untitled");
	}
	else { 
		fEditView->PasteFile(fileName);
		SetTitle(fileName);
	}
	my_window = this;
}


/*------------------------------------------------------------*/

void	TDocWindow::CloseRequested()
{
	if ( this->OKToOpenOrQuit())
	{
		BWindow::CloseRequested();
		my_window = NIL;
		my_app->Quit();
	}
}

//--------------------------------------------------------------
ulong TDocWindow::ComputeChecksum( void )
{
	long 	i, textLen;
	ulong 	sum = 0;
	char	*theText;
	
	theText = this->fEditView->Text();
	textLen = strlen( theText );
	if ( textLen > 0)
		for (i = 0; i <= textLen; i++, sum += theText[i]);
	return sum;
}

//--------------------------------------------------------------
void TDocWindow::Find( void )
{
	char	*searchString;
	
	searchString = this->fEditView->fSearchString;
	if ( GetFileName("Search for:",searchString) )
	{
		this->UpdateIfNeeded();
		strcpy(this->fEditView->fSearchString, searchString);
		this->fEditView->SearchFor( searchString, 1, 1);
	}
}


/*------------------------------------------------------------*/

void	TDocWindow::SaveFile(char* fileName)
{
	FILE*	the_file;
	long	numberOfChars;
	char	*theText;
	char	winTitle[40];
	char	or[40] = "Overwrite ";
	char	*qMark = "?";
	bool	yep = TRUE;
	
	//
	// First, check to see if file exists
	//
	setcurvoldir(this->fVolID, this->fDirID);
	the_file = fopen(fileName,"r");
	if (the_file != NIL)		// File exists
	{
		// Second level check: does file name match window name?
		GetTitle(winTitle);
		if (strcmp(winTitle,fileName))	// No, it's a different file, so check
		{
			strcat(or,fileName);
			strcat(or, qMark);
			yep = AskUser(or);
		}
	fclose(the_file);
	}
	if (yep)
	{
		SetTitle(fileName);
		numberOfChars = this->fEditView->TextLength();
		if ( numberOfChars > 0 )	// Don't save an empty file
		{
			setcurvoldir(this->fVolID, this->fDirID);
			theText = this->fEditView->Text();
			//
			// Opening it for writing removes current contents
			//
			the_file = fopen(fileName, "w");
			fwrite( theText, 1, numberOfChars, the_file );
			fclose(the_file);
		}
		this->checksum = this->ComputeChecksum();
	}
}

//-------------------------------------------------------------
void TDocWindow::PutFile( void )
{
	new MySFDialog("Save file:", TRUE);
}

//-------------------------------------------------------------
void TDocWindow::OpenFile( void )
{
	new MySFDialog("Open file:", FALSE);
}


//-------------------------------------------------------------
bool TDocWindow::Dirty( void )
{
	return this->checksum != this->ComputeChecksum();
}

//--------------------------------------------------------------
bool TDocWindow::OKToOpenOrQuit( void )
{
	if (this->Dirty())
		return AskUser("Throw away changes to this file?");
	else
		return TRUE;
}

/*------------------------------------------------------------*/

void	TDocWindow::CommandReceived(BCommandEvent *an_event)
{
	BRect		r;
	char		state;
	BRect		theBounds;
	char 		title[64] = "";
	long		raskin;
	
	switch(an_event->what) {
		case	ABOUT :
			TellUser("Be Edit, v0.5a");
			break;
		case	REPLACE :
			TellUser("Next release, I promise...");
			break;
		case	JUMPTOP :
			this->fEditView->Select(0,0);
			this->fEditView->ScrollToSelection();
			break;
		case	JUMPBOTTOM :
			raskin = this->fEditView->TextLength();
			this->fEditView->Select(raskin,raskin);
			this->fEditView->ScrollToSelection();
			break;
		case	SEARCH :
			this->Find();
			break;
		case	FINDAGAIN :
			this->fEditView->SearchFor(this->fEditView->fSearchString, 1, 1);
			break;
		case	SAVEAS :
			this->PutFile();
			break;
		case	NEW	:
			if (this->OKToOpenOrQuit())
			{
				fEditView->SelectAll();
				fEditView->Delete();
				SetTitle("untitled");
				this->checksum = this->ComputeChecksum();
			}
			break;
		case	OPEN	:
			if (this->OKToOpenOrQuit())
				this->OpenFile();
			break;
		case	SAVE	:
				GetTitle(title);
				this->SaveFile(title);
			break;
		case	QUIT	:
			if (this->OKToOpenOrQuit())
				my_app->Quit();
			break;
		case	CUT	:
			fEditView->Cut();
			break;
		case	COPY	:
			fEditView->Copy();
			break;
		case	PASTE	:
			fEditView->Paste();
			break;
		case	DELETE	:
			fEditView->Delete();
			break;
		case	SELECTALL:
			fEditView->SelectAll();
			break;
		case	NINE	:
		case	TEN	:
		case	TWELVE	:
		case	FOURTEEN:
		case	EIGHTEEN:
		case	TWENTY	:
		case	TWENTYFOUR:
			SetMenuItemMark(fMenuBar,fTextSize, FALSE);
			SetMenuItemMark(fMenuBar, an_event->what, TRUE);
			fTextSize = an_event->what;
			fEditView->SetFontSize(an_event->what - 11);
			break;
		case	LEFT	:
		case	CENTER	:
		case	RIGHT	:
			SetMenuItemMark(fMenuBar, LEFT + fEditView->Alignment(), FALSE);
			fEditView->SetAlignment(an_event->what - LEFT);
			SetMenuItemMark(fMenuBar, an_event->what, TRUE);
			break;
		case	SINGLE	:
		case	DOUBLE	:
//		case	TRIPLE	:
			SetMenuItemMark(fMenuBar, SINGLE + fEditView->Spacing() - 1, FALSE);
			fEditView->SetSpacing(an_event->what - SINGLE + 1);
			SetMenuItemMark(fMenuBar, an_event->what, TRUE);
			break;
		case	WRAP	:
			state = !fEditView->DoesWordWrap();
			fEditView->SetWordWrap(state);
			SetMenuItemMark(fMenuBar, WRAP, state);
			break;
		case	AUTO	:
			state = !fEditView->DoesAutoindent();
			fEditView->SetAutoindent(state);
			SetMenuItemMark(fMenuBar, AUTO, state);
			break;
	}
};

	
/*------------------------------------------------------------*/

void main(long argc, char* argv[])
{
	TDocWindow*	main_window;
	BRect		a_rect;
	
	my_app = new BApplication();
	
	a_rect.Set( 50, 50, kWindowH + 50, kWindowV + 50);
	if (argc > 1)
		main_window = new TDocWindow(argv[1], &a_rect, TITLED_WINDOW, 0, "Edit");
	else
		main_window = new TDocWindow(NIL, &a_rect, TITLED_WINDOW, 0, "Edit");
	
	main_window->Show();

	my_app->Run();
	
	main_window->Close();
	delete my_app;
}
