
#include <InterfaceKit.h>
#include "stdio.h"
#include "open.h"
#include <TextField.h>
#include <dirview.h>

#define cmdOK 2
#define cmdCancel 1

class myDirView : public DirView
{
public:
	void			myDirView(const BRect *bounds, char *name, long aVolID, long aDirID);
	virtual void		ItemSelected(void *item);
	virtual void		ItemOpened(void *item);
};

void FileOpenWindow::FileOpenWindow( char *title, bool save )
	: BWindow( &BRect(0, 0, 1, 1), title, TITLED_WINDOW, NOT_RESIZABLE | ACCEPTS_FIRST_CLICK)
{
	BView	*view, *v1;
	myDirView	*dv;
	BTextField *t;

	menuBar = NULL;

	LoadFromResource(12);
	SetTitle( title );
	this->fSave = save;	// TRUE if this is a save dialog

	//DoMenu();
	dv = NULL;
	view = FindView("DirRect");
	if(view) {
		long			v, d;
		BRect		bounds;
		BRect		scrollBounds;

		v1 = FindView("Scroll");
		if(v1) {
			v1->GetBounds(&scrollBounds);
			v1->ConvertToParent(&scrollBounds);
		}
		view->GetBounds(&bounds);
		bounds.InsetBy(2, 2);
		bounds.bottom--;
		bounds.OffsetBy(1, 1);
		bounds.right = scrollBounds.left - 1;
		getcurvoldir(&v, &d);
		dv = new myDirView(&bounds, "DirView", v, d);
		view->AddChild(dv);
	}
	if(dv)
		dv->Invalidate();
	Show();
	AddKeyCommand('\n', 0, cmdOK);
	AddKeyCommand('\r', 0, cmdOK);
	AddKeyCommand(27, 0, cmdCancel);
	if(dv)
		dv->Invalidate();
	t = (BTextField *) FindView("Entry");
	if(t)
		t->MakeFocus(TRUE);
}

void AddItems(TMenu *m)
{
	long		v, d;
	FileEntry	info;
	DirEntry	*dir = (DirEntry *)&info;
	
	getcurvoldir(&v, &d);
	
	get_ind_info(v, d, -1, &info);
	if(strcmp(dir->DirName, "ROOT")) {
		chdir("..");
		AddItems(m);
	}
	printf("%s\n", dir->DirName);
	m->add_item(dir->DirName, (int) d+32);
	setcurvoldir(v, d);
}

void FileOpenWindow::DoMenu(void)
{
#ifdef MENUS_WORK	
	struct vol_info vol;
	long		v, d;
	BRect	bounds;
	BView	*view;

	FindView("DirRect")->GetBounds(&bounds);
	FindView("DirRect")->ConvertToScreen(&bounds);
	ConvertFromScreen(&bounds);
	bounds.top -= 17;
	bounds.bottom = bounds.top + 13;
	bounds.left = bounds.left + (bounds.Width()/2) - 15;
	bounds.right = bounds.left + 15;

	view = FindView("Canvas");
	if(view) {
		ConvertToScreen(&bounds);
		view->ConvertFromScreen(&bounds);
	}
	if(!menuBar)
		menuBar = new TMenuBar( "chDir", &bounds, WILL_DRAW | FOLLOW_LEFT_TOP_RIGHT );
	if(!menuBar) {
		BRaise();
		return;
	}
	if(view)
		view->AddChild(menuBar);
	BAssert(view);
	getcurvoldir(&v, &d);
	memset(&vol, '\0', sizeof(vol));
	vol.id = v;
	get_vol_info(&vol);
	printf("%s\n", vol.name);
	if(m) {
		delete m;
		m = NULL;
	}
	m = new TMenu( (char *) vol.name, menuBar);
	AddItems(m);
	setcurvoldir(v, d);
#endif
}

void FileOpenWindow::~FileOpenWindow(void)
{
}

void FileOpenWindow::CommandReceived(BCommandEvent *an_event)
{
	long msg = an_event->what;
	BTextField *t;
	myDirView	*m;
	FileEntry	info;
	
	switch(msg) {
		case cmdOK:
			t = (BTextField *) FindView("Entry");
			if(t && t->Text() && strlen(t->Text())) {
				long		v, d;
				static char	name[33];
				static char path[128];
				char	*p;
				
				m = (myDirView *) FindView("DirView");
				getcurvoldir(&v, &d);
				p = t->Text();
				p = &p[strlen(p)-1];
				while(p > t->Text() && *p != '/' && *p != ':')
					p--;
				if(p == t->Text() && *p != '/' && *p != ':') {
					path[0] = '\0';
					strcpy(name, p);
				}
				else {
					char	c = *(p+1);
					
					*(p+1) = '\0';
					strcpy(path, t->Text());
					*(p+1) = c;
					c = *p;
					strcpy(name, p+1);
					*p = c;
					chdir(path);
					getcurvoldir(&v, &d);
				}
				if ( this->fSave )
				{
					FileSaved(v, d, name);
					return;
				}
				if( get_named_info(v, d, name, &info) )
					return;
				if(info.FileType != -1)
					FileOpened(v, d, name);
				else {
					chdir(name);
					getcurvoldir(&v, &d);
					m->ChangeTo(v, d);
					return;
				}
			}
			return;
		case cmdCancel:
			Close();
			return;
	}
}

void FileOpenWindow::FileSelected(long volID, long dirID, char *fileName)
{
	BTextField *t;

	t = (BTextField *) FindView("Entry");
	if(t)
		t->SetText(fileName);
}

void FileOpenWindow::FileOpened(long volID, long dirID, char *fileName)
{
	BTextField *t;

	t = (BTextField *) FindView("Entry");
	if(t) {
		t->SetText(fileName);
		t->Select(0, strlen(fileName));
	}
}

void FileOpenWindow::FileSaved(long volID, long dirID, char *fileName)
{
//
// Subclass must implement
//
}

void myDirView::myDirView(const BRect *bounds, char *name, long aVolID, long aDirID)
	: DirView(bounds, name, aVolID, aDirID)
{
}

void myDirView::ItemSelected(void *item)
{
	FileEntry *p = (FileEntry *) item;
	long v, d;

	if(p->FileType == -1)
		DirView::ItemSelected(item);
	getcurvoldir(&v, &d);
	((FileOpenWindow *) Window())->FileSelected(v, d, p->FileName);
}

void myDirView::ItemOpened(void *item)
{
	FileEntry *p = (FileEntry *) item;
	long v, d;

	getcurvoldir(&v, &d);
	if(p->FileType == -1) {
		((FileOpenWindow *) Window())->FileSelected(0, 0, "");
		DirView::ItemOpened(item);
		((FileOpenWindow *) Window())->DoMenu();
		return;
	}
	if (((FileOpenWindow *) Window())->fSave)
		((FileOpenWindow *) Window())->FileSaved(v, d, p->FileName);
	else
		((FileOpenWindow *) Window())->FileOpened(v, d, p->FileName);
}
